;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; GENSYM v. 3.1, 11 May 2013.                                                  ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; This procedure generates symbols for plots.                                  ;
;                                                                              ;
; Positional parameters:                                                       ;
; type:   0 for a circle (default value), 1 for a triangle, 2 for an inverted  ;
;          triangle, 3 for a square, 4 for a diamond, 5 for a simple cross,    ;
;          6 for a pentagon, 7 for an hexagon, 8 for a simple star, 9 for a    ;
;          star of David, 10 for a complex cross, 11 for a Sun, 12 for a       ;
;          crescent Moon, 13 for an Earth symbol, 14 for a heart, 20 for a     ;
;          vertical line, 21 for a horizontal line, 30 for a lower limit,      ;
;          31 for an upper limit, 32 for a right limit, 33 for a left limit,   ;
;          and 34-37 for limits in the four quadrants.                         ;
; Keyword parameters:                                                          ;
; NOFILL: Flag to not fill the symbol.                                         ;
; THICK:  Line thickness (default value, 1.0).                                 ;
;                                                                              ;
; Version history:                                                             ;
; v0.1:   General programming changes                                          ;
;         Options 6 to 13 added.                                               ;
; v0.2:   THICK option added.                                                  ;
; v1.0:   General clean up.                                                    ;
; v2.0:   English version.                                                     ;
; v2.1:   IDLdoc documentation.                                                ;
;         Addition of new symbols and displacement of lines and limits to      ;
;          upper values.                                                       ;
; v2.5:   Version system update.                                               ;
; v3.0:   Adapted to SINGLEPLOT overall changes.                               ;
; v3.1:   Only version change.                                                 ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
PRO GENSYM, type, NOFILL=nofill, THICK=thick
ON_ERROR, 1
valid = [INDGEN(15),20+INDGEN(2),30+INDGEN(8),40+INDGEN(8)]
IF N_ELEMENTS(type)  EQ 0 THEN type  = 0
IF KEYWORD_SET(THICK) EQ 0 THEN thick = 1.0
type2 = type
IF type2 GE 100 THEN type2 = type2 - 100
dist = ABS(valid-type2)
mindist = MIN(dist)
IF mindist NE 0 THEN MESSAGE, STRING(type) + ' is a non-valid symbol type'
IF type2 LT 30 THEN BEGIN
 CASE type2 OF
;
; Crculo
;
  0:  BEGIN
       a = FINDGEN(49) * (!PI*2/48.)
       r = REPLICATE(0.8,49)
      END
;
; Tringulo
;
  1:  BEGIN
       a = FINDGEN(4) * (!PI*2/3.) + !PI/2.
       r = REPLICATE(0.8,4)
      END
;
; Tringulo invertido
;
  2:  BEGIN
       a = FINDGEN(4) * (!PI*2/3.) - !PI/2.
       r = REPLICATE(0.8,4)
      END
;
; Cuadrado
;
  3:  BEGIN
       a = FINDGEN(6) * (!PI*2/4.) + !PI/4.
       r = REPLICATE(0.8,6)
      END
;
; Diamante
;
  4:  BEGIN
       a = FINDGEN(5) * (!PI*2/4.) 
       r = REPLICATE(0.8,5)
      END
;
; Cruz
;
  5:  BEGIN
       x = [ 0.8, 0.2, 0.2,-0.2,-0.2,-0.8,-0.8,-0.2,-0.2, 0.2, 0.2, 0.8, 0.8]
       y = [ 0.2, 0.2, 0.8, 0.8, 0.2, 0.2,-0.2,-0.2,-0.8,-0.8,-0.2,-0.2, 0.2]
       r = SQRT(x^2+y^2)
       a = ATAN(y,x)
      END
;
; Pentgono
;
  6:  BEGIN
       a = FINDGEN(6) * (!PI*2/5.) + !PI/2.
       r = REPLICATE(0.8,6)
      END
;
; Hexgono
;
  7:  BEGIN
       a = FINDGEN(7) * (!PI*2/6.)
       r = REPLICATE(0.8,7)
      END
;
; Estrella
;
  8:  BEGIN
       a = FINDGEN(11) * (!PI*2/10.) + !PI/2.
       r = FLTARR(11)
       r[2*INDGEN(6)]   = 0.9
       r[2*INDGEN(5)+1] = 0.3
      END
;
; Estrella de David
;
  9:  BEGIN
       a1 = FINDGEN(13) * (!PI*2/12.)
       a2 = FINDGEN(6)  * (!PI*2/6.)  + !PI/3.
       a  = [a1,a2]
       r = FLTARR(19)
       r[2*INDGEN(7)]   = 0.5
       r[2*INDGEN(6)+1] = 0.5*SQRT(3)
       r[13:18      ]   = 0.5
      END
;
; Cruz compleja
;
  10: BEGIN
       x = [ 0.8, 0.8, 0.6, 0.6, 0.1, 0.1, 0.4, 0.4, 0.0]
       y = [ 0.0, 0.4, 0.4, 0.1, 0.1, 0.6, 0.6, 0.8, 0.8]
       r = SQRT(x^2+y^2)
       a = ATAN(y,x)
       r = [r,r        ,r        ,r        ]
       a = [a,a+0.5*!PI,a+1.0*!PI,a+1.5*!PI]
      END
;
; Sol
;
  11: BEGIN
       a = FINDGEN(21) * (!PI*2/20.) + !PI/2.
       r = FLTARR(21)
       r[2*INDGEN(11)]   = 0.9
       r[2*INDGEN(10)+1] = 0.3
      END
;
; Luna
;
  12: BEGIN
       a = FINDGEN(25) * (!PI*2/48.) + !PI/2.
       a = [a,REVERSE(a[0:23])]
       r = FLTARR(49)
       r[0:23]  = 0.8
       r[24:48] = 0.8*(0.5*(1+((FINDGEN(25)-12.)/12.)^2))
       x = r*COS(a) + 0.3
       y = r*SIN(a) + 0.3
       r = SQRT(x^2+y^2)
       a = ATAN(y,x)
      END
;
; Tierra 
;
  13: BEGIN
       a = FINDGEN(45) * (!PI*2/44.) + !PI/2.
       r = REPLICATE(0.8,45)
       a = [a,1.5*!PI,0.0,0.0,!PI]
       r = [r,    0.8,0.0,0.8,0.8]
      END
;
; Corazn
;
  14: BEGIN
       b = ATAN(3.0,4.0)
       r = REPLICATE(0.8*((5./3.)/4.0),21)
       a = FINDGEN(21)*((!PI+b)/20) - b
       x = [ 0.0,r*COS(a) + 0.8*(1.5/4.0)]
       y = [-0.8,r*SIN(a) + 0.8*(1.0/4.0)]
       x = [x,-1.0*REVERSE(x)]
       y = [y,     REVERSE(y)]
       r = SQRT(x^2+y^2)
       a = ATAN(y,x)
      END
;
; Raya vertical
;
  20: BEGIN
       a = FINDGEN(2) * (!PI) + !PI/2.
       r = REPLICATE(0.8,2)
      END
;
; Raya horizontal
;
  21: BEGIN
       a = FINDGEN(2) * (!PI) 
       r = REPLICATE(0.8,2)
      END
 ENDCASE
;
; Definimos el smbolo
;
 IF KEYWORD_SET(NOFILL) OR type2 EQ 13 OR type2 GE 20 THEN $
    USERSYM, r*COS(a), r*SIN(a), THICK=thick          ELSE $
    USERSYM, r*COS(a), r*SIN(a), THICK=thick, /FILL
ENDIF ELSE BEGIN
 CASE type2 OF
  30: BEGIN
;       x=[ 0.0, 0.0, 0.5,-0.5, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0]
;       y=[ 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 2.0, 1.0, 1.0]
       x=[ 0.1, 0.1, 0.5, 0.5,-0.5,-0.5,-0.1,-0.1,-0.5, 0.0, 0.5, 0.1]
       y=[ 1.0, 0.1, 0.1,-0.1,-0.1, 0.1, 0.1, 1.0, 1.0, 2.0, 1.0, 1.0]
      END
  31: BEGIN
;       x=[ 0.0, 0.0, 0.5,-0.5, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0]
;       y=[-1.0, 0.0, 0.0, 0.0, 0.0,-1.0,-1.0,-2.0,-1.0,-1.0]
       x=[ 0.1, 0.1, 0.5, 0.5,-0.5,-0.5,-0.1,-0.1,-0.5, 0.0, 0.5, 0.1]
       y=[-1.0,-0.1,-0.1, 0.1, 0.1,-0.1,-0.1,-1.0,-1.0,-2.0,-1.0,-1.0]
      END
  32: BEGIN
;       x=[-1.0, 0.0, 0.0, 0.0, 0.0,-1.0,-1.0,-2.0,-1.0,-1.0]
;       y=[ 0.0, 0.0, 0.5,-0.5, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0]
       x=[-1.0,-0.1,-0.1, 0.1, 0.1,-0.1,-0.1,-1.0,-1.0,-2.0,-1.0,-1.0]
       y=[ 0.1, 0.1, 0.5, 0.5,-0.5,-0.5,-0.1,-0.1,-0.5, 0.0, 0.5, 0.1]
      END
  33: BEGIN
;       x=[ 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 2.0, 1.0, 1.0]
;       y=[ 0.0, 0.0, 0.5,-0.5, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0]
       x=[ 1.0, 0.1, 0.1,-0.1,-0.1, 0.1, 0.1, 1.0, 1.0, 2.0, 1.0, 1.0]
       y=[ 0.1, 0.1, 0.5, 0.5,-0.5,-0.5,-0.1,-0.1,-0.5, 0.0, 0.5, 0.1]
      END
  34: BEGIN
;       x=[ 0.0, 0.0, 1.0, 1.0, 2.0, 1.0, 1.0, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0]
;       y=[ 1.0, 0.0, 0.0,-0.5, 0.0, 0.5, 0.0, 0.0, 1.0, 1.0, 2.0, 1.0, 1.0]
       x=[-0.1,-0.1, 1.0, 1.0, 2.0, 1.0, 1.0, 0.1, 0.1, 0.5, 0.0,-0.5,-0.1]
       y=[ 1.0,-0.1,-0.1,-0.5, 0.0, 0.5, 0.1, 0.1, 1.0, 1.0, 2.0, 1.0, 1.0]
      END
  35: BEGIN
;       x=[ 0.0, 0.0,-1.0,-1.0,-2.0,-1.0,-1.0, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0]
;       y=[ 1.0, 0.0, 0.0,-0.5, 0.0, 0.5, 0.0, 0.0, 1.0, 1.0, 2.0, 1.0, 1.0]
       x=[ 0.1, 0.1,-1.0,-1.0,-2.0,-1.0,-1.0,-0.1,-0.1, 0.5, 0.0,-0.5, 0.1]
       y=[ 1.0,-0.1,-0.1,-0.5, 0.0, 0.5, 0.1, 0.1, 1.0, 1.0, 2.0, 1.0, 1.0]
      END
  36: BEGIN
;       x=[ 0.0, 0.0,-1.0,-1.0,-2.0,-1.0,-1.0, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0]
;       y=[-1.0, 0.0, 0.0,-0.5, 0.0, 0.5, 0.0, 0.0,-1.0,-1.0,-2.0,-1.0,-1.0]
       x=[ 0.1, 0.1,-1.0,-1.0,-2.0,-1.0,-1.0,-0.1,-0.1, 0.5, 0.0,-0.5, 0.1]
       y=[-1.0, 0.1, 0.1, 0.5, 0.0,-0.5,-0.1,-0.1,-1.0,-1.0,-2.0,-1.0,-1.0]
      END
  37: BEGIN
;       x=[ 0.0, 0.0, 1.0, 1.0, 2.0, 1.0, 1.0, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0]
;       y=[-1.0, 0.0, 0.0,-0.5, 0.0, 0.5, 0.0, 0.0,-1.0,-1.0,-2.0,-1.0,-1.0]
       x=[-0.1,-0.1, 1.0, 1.0, 2.0, 1.0, 1.0, 0.1, 0.1, 0.5, 0.0,-0.5,-0.1]
       y=[-1.0, 0.1, 0.1, 0.5, 0.0,-0.5,-0.1,-0.1,-1.0,-1.0,-2.0,-1.0,-1.0]
      END
  40: BEGIN
       x=[ 0.0, 0.0, 1.0,-1.0, 0.0, 0.0,-0.5, 0.0, 0.5, 0.0]
       y=[ 1.0, 0.0, 0.0,-0.0, 0.0, 1.0, 1.0, 2.0, 1.0, 1.0]
      END
  41: BEGIN
       x=[ 0.0, 0.0, 1.0,-1.0, 0.0, 0.0,-0.5, 0.0, 0.5, 0.0]
       y=[-1.0, 0.0, 0.0, 0.0, 0.0,-1.0,-1.0,-2.0,-1.0,-1.0]
      END
  42: BEGIN
       x=[-1.0, 0.0, 0.0, 0.0, 0.0,-1.0,-1.0,-2.0,-1.0,-1.0]
       y=[ 0.0, 0.0, 1.0,-1.0, 0.0,-0.0,-0.5, 0.0, 0.5, 0.0]
      END
  43: BEGIN
       x=[ 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 2.0, 1.0, 1.0]
       y=[ 0.0, 0.0, 1.0,-1.0, 0.0, 0.0,-0.5, 0.0, 0.5, 0.0]
      END
  44: BEGIN
       x=[ 0.0, 0.0, 1.0, 1.0, 2.0, 1.0, 1.0, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0]
       y=[ 1.0, 0.0, 0.0, 0.5, 0.0, 0.5, 0.0, 0.0, 1.0, 1.0, 2.0, 1.0, 1.0]
      END
  45: BEGIN
       x=[ 0.0, 0.0,-1.0,-1.0,-2.0,-1.0,-1.0, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0]
       y=[ 1.0, 0.0, 0.0,-0.5, 0.0, 0.5, 0.0, 0.0, 1.0, 1.0, 2.0, 1.0, 1.0]
      END
  46: BEGIN
       x=[ 0.0, 0.0,-1.0,-1.0,-2.0,-1.0,-1.0, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0]
       y=[-1.0, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0, 0.0,-1.0,-1.0,-2.0,-1.0,-1.0]
      END
  47: BEGIN
       x=[ 0.0, 0.0, 1.0, 1.0, 2.0, 1.0, 1.0, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0]
       y=[-1.0, 0.0, 0.0, 0.5, 0.0,-0.5, 0.0, 0.0,-1.0,-1.0,-2.0,-1.0,-1.0]
      END
 ENDCASE
 IF KEYWORD_SET(NOFILL)        THEN $
  USERSYM, x, y, THICK=thick   ELSE $
  USERSYM, x, y, THICK=thick, /FILL 
ENDELSE
END

