;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; LEGEND v. 3.1, 11 May 2013.                                                  ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; This procedure writes legends (which are fictitious, of course) for          ;
;  SINGLEPLOT.                                                                 ;
;                                                                              ;
; Positional parameters:                                                       ;
; legend:    Legend structure.                                                 ;
; plot:      Plot structure.                                                   ;
; cfile:     Color file structure.                                             ;
; color:     Array with the color for each data set.                           ;
; linestyle: Array with the line styles for each data set.                     ;
; psym:      Array with the symbols for each data set.                         ;
; thick:     Array with the line thicknesses for each data set.                ;
; symsize:   Array with the symbol sizes for each data set.                    ;
; nofill:    Array with the no-fill flags for each data set.                   ;
; errort:    Error type array: 0 for no error, 1 for errors in x, 2 for errors ;
;             in y, 3 for errors in x and y, 4 for errors with an arbitrary    ;
;             direction, and 5 for text-drawing.                               ;
;                                                                              ;
; Version history:                                                             ;
; v0.1:      symsize changed to positional parameter.                          ;
; v0.2:      NOFILL flag added.                                                ;
;            Bug related to THICK fixed.                                       ;
; v1.0:      General clean up.                                                 ;
; v1.1:      Possibility of empty entries included with legend[i] =''.         ;
; v2.0:      English version.                                                  ;
; v2.1:      IDLdoc documentation.                                             ;
; v2.5:      Version system update.                                            ;
; v3.0:      Adapted to SINGLEPLOT overall changes.                            ;
; v3.1:      NOBORDER added.                                                   ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
PRO LEGEND, legend, pplot, cfile, $
	    color, linestyle, psym, symsize, thick, nofill, errort
            
ON_ERROR, 1
;
; Inicializacin de variables
;
ltype        = SIZE(legend.text, /N_DIM)
ltype        = 1*(ltype LE 1) + ltype*(ltype GT 1)
IF ltype EQ 1 THEN BEGIN
 n           = N_ELEMENTS(legend.text)
 aux         = WHERE(legend.text NE '',nreal)
 IF nreal EQ 0 THEN MESSAGE, 'Legend without text'
 legendreal  = legend.text[WHERE(legend.text NE '')]
 legendtext2 = legend.text
 lengthv     = LENGTHV2(legend.text)
ENDIF ELSE BEGIN
 n           = N_ELEMENTS(legend.text[*,0])
 aux         = WHERE(legend.text[*,1] NE '',nreal)
 IF nreal NE n THEN MESSAGE, 'Incomplete SINGLECONTOUR legend'
 legendreal  = legend.text
 legendtext2 = legend.text[*,0] + ' ' + legend.text[*,1] + ' ' + legend.text[*,2]
 lengthv     = LENGTHV2(STRTRIM(legendtext2,2))
ENDELSE
IF (ABS(N_ELEMENTS(color )-n) + ABS(N_ELEMENTS(linestyle)-n) + $
    ABS(N_ELEMENTS(psym  )-n) + ABS(N_ELEMENTS(symsize  )-n)     + $
    ABS(N_ELEMENTS(thick )-n) + ABS(N_ELEMENTS(nofill   )-n)     + $
    ABS(N_ELEMENTS(errort)-n)) NE 0 THEN PRINT,                $
    'Warning: Not all vectors in legend have the same number of components'
COPY_STRUC, legend, legend, EXCEPT_TAGS=['xright','yup'],  $
	     STRUC_ADD={xright:legend.xleft+legend.xsize, $
	                   yup:legend.ydown+legend.ysize}
;
; Obtenemos los parmetros geomtricos para colocar los elementos de la leyenda
; mx, extrax, my, sx, tx y ty estn normalizados respecto a la leyenda
;
mx        = MAX([0.0075/legend.xsize,MIN([0.075,0.030/legend.xsize])]) 
my        = 0.05
extrax    = mx 
minpsym   = MIN(psym)
IF minpsym LE 0 THEN sx = 0.20 ELSE sx = 0.05
FOR i=0,n-1 DO IF sx EQ 0.05 AND errort[i] NE 0 AND errort[i]      NE 2  AND $
	                         errort[i] NE 4 AND legendtext2[i] NE '' THEN sx = 0.10
tx        =  1.0 - 2*mx - extrax - sx
ty        = (1.0 - 2*my)/nreal
COPY_STRUC, legend, legend, EXCEPT_TAGS=['xsizecm','ysizecm'],  $
	     STRUC_ADD={xsizecm:pplot.psxsize*pplot.xsize*legend.xsize, $
                        ysizecm:pplot.psysize*pplot.ysize*legend.ysize}
maxl      = MAX(lengthv)
charsize  = tx*legend.xsizecm*5.0/maxl          ; Para charsize=1: 5.0 carac./cm
IF ty*legend.ysizecm LT 0.4*charsize THEN BEGIN ; Para charsize=1: 0,4 cm altura
 charsize = ty*legend.ysizecm/0.4
 PRINT, 'Warning: Legend box too short in y'
ENDIF
;
; Dibujamos la caja
;
xmin = pplot.box[0]+pplot.extraxl+pplot.xsize*legend.xleft
xmax = pplot.box[0]+pplot.extraxl+pplot.xsize*legend.xright
ymin = pplot.box[1]+pplot.extrayl+pplot.ysize*legend.ydown
ymax = pplot.box[1]+pplot.extrayl+pplot.ysize*legend.yup     
IF legend.clear NE 0 THEN $
 POLYFILL,  [xmin, xmin, xmax, xmax, xmin], $
            [ymin, ymax, ymax, ymin, ymin], COLOR=255, /NORMAL
IF legend.noborder EQ 0 THEN BEGIN
 PLOTS, [xmin, xmin], [ymin, ymax], /NORMAL, COLOR=cfile.def
 PLOTS, [xmin, xmax], [ymax, ymax], /NORMAL, COLOR=cfile.def
 PLOTS, [xmax, xmax], [ymax, ymin], /NORMAL, COLOR=cfile.def
 PLOTS, [xmax, xmin], [ymin, ymin], /NORMAL, COLOR=cfile.def
ENDIF
;
; Escribimos el texto
;
ypos0  = pplot.box[1]+pplot.extrayl+pplot.ysize*(legend.yup-legend.ysize*(my+ty*(0.5+FINDGEN(nreal))+0.1*charsize/legend.ysizecm))
aux    = mx+sx+extrax
IF ltype EQ 1 THEN BEGIN
 xpos0 = pplot.box[0]+pplot.extraxl+pplot.xsize*(legend.xleft+legend.xsize*(aux))
 XYOUTS, xpos0, ypos0, legendreal, CHARTHICK=pplot.charthick, CHARSIZE=charsize, /NORMAL, COLOR=cfile.def, FONT=pplot.font
ENDIF ELSE BEGIN
 xpos0 = pplot.box[0]+pplot.extraxl+pplot.xsize*(legend.xleft+legend.xsize*(aux + 0.5*(1.0-aux-mx) - mx))
 XYOUTS, xpos0, ypos0, legendreal[*,0], $
	 CHARTHICK=pplot.charthick, CHARSIZE=charsize, /NORMAL, COLOR=cfile.def, FONT=pplot.font, ALIGNMENT=1.0
 xpos0 = pplot.box[0]+pplot.extraxl+pplot.xsize*(legend.xleft+legend.xsize*(aux + 0.5*(1.0-aux-mx)     ))
 XYOUTS, xpos0, ypos0, legendreal[*,1], $
	 CHARTHICK=pplot.charthick, CHARSIZE=charsize, /NORMAL, COLOR=cfile.def, FONT=pplot.font, ALIGNMENT=0.5
 xpos0 = pplot.box[0]+pplot.extraxl+pplot.xsize*(legend.xleft+legend.xsize*(           1.0    -mx      ))
 XYOUTS, xpos0, ypos0, legendreal[*,2], $
	 CHARTHICK=pplot.charthick, CHARSIZE=charsize, /NORMAL, COLOR=cfile.def, FONT=pplot.font, ALIGNMENT=1.0
ENDELSE
;
; Dibujamos el tipo de grfica
;
xpos1 = pplot.box[0]+pplot.extraxl+pplot.xsize*(legend.xleft+legend.xsize* mx)
xpos2 = pplot.box[0]+pplot.extraxl+pplot.xsize*(legend.xleft+legend.xsize*(mx+0.5*sx))
xpos3 = pplot.box[0]+pplot.extraxl+pplot.xsize*(legend.xleft+legend.xsize*(mx+sx))
ypos  = pplot.box[1]+pplot.extrayl+pplot.ysize*(legend.yup  -legend.ysize*(my+ty*(0.5+FINDGEN(nreal))))
ii = 0
FOR i=0,n-1 DO BEGIN
 IF legend.text[i] NE '' THEN BEGIN
;
; Caso 1: Lnea
;
  IF psym[i] LE 0 AND errort[i] LE 5 THEN BEGIN
   PLOTS, [xpos1,xpos3], [ypos[ii],ypos[ii]], /NORMAL, $
          COLOR=color[i], LINESTYLE=linestyle[i], THICK=thick[i]
  ENDIF
;
; Caso 2: Smbolo o histograma
;
  IF psym[i] NE 0 AND errort[i] LE 5 THEN BEGIN
   xx = FLTARR(1) & yy = FLTARR(1) 
   aux   = CONVERT_COORD(xpos2,ypos[ii],/NORMAL,/TO_DATA)
   xx[0] = aux[0] & yy[0] = aux[1]
   IF psym[i] NE 10 THEN BEGIN
    IF ABS(psym[i]) GE 100 THEN BEGIN
     GENSYM, ABS(psym[i])-100, NOFILL=nofill[i], THICK=thick[i]
     auxpsym = 8
    ENDIF ELSE IF psym[i] EQ 11 OR psym[i] EQ 12 THEN BEGIN
     GENSYM, 103, NOFILL=0, THICK=1.0
     auxpsym = 8
    ENDIF ELSE BEGIN
     auxpsym = ABS(psym[i])
    ENDELSE
    OPLOT, xx, yy, PSYM=auxpsym, COLOR=color[i], SYMSIZE=symsize[i], $
     LINESTYLE=linestyle[i], THICK=thick[i]
   ENDIF
   IF psym[i] EQ 10 OR psym[i] EQ 12 THEN BEGIN
    GENSYM, 103, NOFILL=1, THICK=thick[i]
    colori = color[i]*(psym[i] EQ 10) +  $
            cfile.def*(psym[i] EQ 12)
    OPLOT, xx, yy, PSYM=8, COLOR=colori, SYMSIZE=symsize[i], $
     LINESTYLE=linestyle[i], THICK=thick[i]
   ENDIF
  ENDIF
;
; Caso 3: Barras de error
;
  IF errort[i] GE 1 AND errort[i] LE 5 THEN BEGIN
; Barra horizontal
   IF errort[i] NE 2 AND errort[i] NE 4 THEN BEGIN
    lbar1 = 0.50*pplot.xsize*legend.xsize*sx
    lbar2 = 0.15*pplot.ysize*legend.ysize*ty
    PLOTS, [xpos2    - lbar1, xpos2    - lbar1], $
           [ypos[ii] - lbar2, ypos[ii] + lbar2], /NORMAL, COLOR=color[i], $
           THICK=thick[i]
    PLOTS, [xpos2    - lbar1, xpos2    + lbar1], $
           [ypos[ii]        , ypos[ii]        ], /NORMAL, COLOR=color[i], $
           THICK=thick[i]
    PLOTS, [xpos2    + lbar1, xpos2    + lbar1], $
           [ypos[ii] - lbar2, ypos[ii] + lbar2], /NORMAL, COLOR=color[i], $
           THICK=thick[i]
   ENDIF
; Barra vertical
   IF errort[i] NE 1 THEN BEGIN
    lbar1 = 0.40*pplot.ysize*legend.ysize*ty
    lbar2 = 0.15*pplot.xsize*legend.xsize*sx
    PLOTS, [xpos2    - lbar2, xpos2    + lbar2], $
           [ypos[ii] - lbar1, ypos[ii] - lbar1], /NORMAL, COLOR=color[i], $
           THICK=thick[i]
    PLOTS, [xpos2           , xpos2           ], $
           [ypos[ii] - lbar1, ypos[ii] + lbar1], /NORMAL, COLOR=color[i], $
           THICK=thick[i]
    PLOTS, [xpos2    - lbar2, xpos2    + lbar2], $
           [ypos[ii] + lbar1, ypos[ii] + lbar1], /NORMAL, COLOR=color[i], $
           THICK=thick[i]
   ENDIF
  ENDIF
;
; Caso 4: Flechas
;
  IF errort[i] EQ 6 THEN BEGIN
   dx = 0.50*pplot.xsize*legend.xsize*sx
   dy = 0.15*pplot.ysize*legend.ysize*ty
   ARROW2, xpos2-dx, ypos[ii], xpos2+dx, ypos[ii], COL=color[i], /NORMAL
  ENDIF
;
; Caso 5: Texto
;
  IF errort[i] EQ 7 THEN BEGIN
   XYOUTS, xpos2, ypos0[ii], 'text', CHARSIZE=0.7*charsize, /NORMAL, COLOR=cfile.def, $
        FONT=pplot.font, ALIGNMENT=0.5
  ENDIF
  ii = ii + 1
 ENDIF
ENDFOR
END

