;+
; MULTIGRAPH is a wrapper that generates multiplot PostScript files. It can be
;  used with SINGLEPLOT, SINGLECONTOUR, or with any other IDL procedure.
; See the <a HREF="../multigraph.pro">full procedure</a><BR>
; @examples <PRE>
;
;  A simple call to MULTIGRAPH is:
;   x1    = FINDGEN(101)
;   y1    = x1 + x1^2 - 0.01*x1^3 + 10*RANDOMN(seed,101)
;   x     = -50. + FINDGEN(101)
;   y     = 100. + FINDGEN(51)
;   xx    = x # (1.+FLTARR(51))
;   yy    = (1.+FLTARR(101)) # y
;   f     = 5*(SIN(0.08*xx)*SIN(0.12*yy))^2+5*(SIN(0.05*(xx+yy)))^2 + 0.3*RANDOMN(seed,101,51)
;   g1    = {type:'singlecontour', v1:{f:f}}
;   g2    = {type:'singleplot',    v1:{x:x1, y:y1}}
;   MULTIGRAPH, "test15a.ps", g1, g2
;  See the <a HREF="jmaplot_overview.html#tutorial" TARGET="parent">jmaplot tutorial</a> for additional examples. </PRE>
; @version v. 3.1, 11 May 2013.                                         <BR>
; @history <PRE>                                                        
;
;  v0.1: V1 to V5 variables and PASSIJ variables introduced.              
;        LANDSCAPE flag introduced.                               
;  v1.0: General clean up.                                                
;  v1.1: Default values for CFILE and CTABLE become 'colorjma.tbl' and    
;         37, respectively.                                               
;  v2.0: English version.
;  v2.1: IDLdoc documentation.
;        CTABLE incompatibilities eliminated.
;        Variables are received by auxiliary files always using _REF_EXTRA.
;         This makes PASSIJ obsolete and allows for the easy inclusion of up
;         to V9, but produces an incompatibilty with previous auxiliary
;         procedures which is easily solved by adding ',_REF_EXTRA=ref_extra'
;         to their definitions.
;        Fix to the small font problem when using MULTIGRAPH (NPX).
;  v2.2: OPENHTML introduced.
;        Upgrade to IDLdoc documentation.
;  v2.3: To ensure compatibility, use _EXTRA instead of _REF_EXTRA.
;        I and J changed to II and JJ due to keyword problems with CONTOUR.
;        Windows compatibility enhanced.
;  v2.4: Error with _EXTRA fixed. 
;  v2.5: Version system update.
;        QUIET flag added.   
;  v3.0: Full reprogramming: previous syntax obsolete. 
;        Use of structures and adaptation to SINGLEPLOT and SINGLECONTOUR changes.
;        Full positioning control.
;        Need for auxiliary procedures eliminated.
;        New wagoncircle mode.
;        CMYK PS output added.
;        HTML pages moved to local directory.
;        General changes.       
;  v3.1: Only version change. </PRE>
; @author Jess Maz Apellniz, IAA   <BR>
; @param   psname {in}{required}{type=string} Postscript file name. <BR>
; @param   g1 {in}{required}{type=structure} First graph structure. 
;           Possible tags: <PRE>
;           TYPE:      Required string. It can be 'singleplot', 'singlecontour', or
;                       any call to a valid IDL procedure.
;           V1:        (First) data structure for SINGLEPLOT and SINGLECONTOUR.
;           V1...V60:  Additional graph structures for SINGLEPLOT.
;           ...:       The rest of the SINGLEPLOT/SINGLECONTOUR keywords.
; @param   g2 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g3 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g4 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g5 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g6 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g7 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g8 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g9 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g10 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g11 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g12 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g13 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g14 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g15 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g16 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g17 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g18 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g19 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g20 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g21 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g22 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g23 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g24 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g25 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g26 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g27 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g28 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g29 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g30 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g31 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g32 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g33 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g34 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g35 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g36 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g37 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g38 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g39 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g40 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g41 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g42 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g43 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g44 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g45 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g46 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g47 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g48 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g49 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g50 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g51 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g52 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g53 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g54 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g55 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g56 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g57 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g58 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g59 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g60 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g61 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g62 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g63 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g64 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g65 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g66 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g67 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g68 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g69 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g70 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g71 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g72 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g73 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g74 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g75 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g76 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g77 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g78 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g79 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g80 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g81 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g82 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g83 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g84 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g85 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g86 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g87 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g88 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g89 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g90 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g91 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g92 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g93 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g94 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g95 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g96 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g97 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g98 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g99 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @param   g100 {in}{optional}{type=structure} Additional graph structure. See g1 for tags. <BR>
; @keyword MULTI {in}{optional}{type=structure} Structure used to supress default values. 
;           The possible tags are: <PRE>
;           FREE:        Flag for free positioning mode. If activated,
;                         xc, yc, xcenter, ycenter) in each graph are normalized
;                         with respect to the full PS file.
;           BOXES:       4xngraphs float array with the xleft, ydown, xright, yup
;                         normal coordinates to place the graphs. Note that 
;                         xc, yc, xcenter, ycenter in each graph are normalized
;                         to these boxes if FREE is not activated.
;           XYN:         2-D array with the number of graphs in the x and y direction
;                         that overrides the default choice. Ignored if FREE is
;                         activated.
;           ASPECT:      By default, each graph has an aspect ratio of 1.0. In order
;                         to change this, set aspect to:
;                          >0.0: The value of the aspect ratio of each graph.
;                          =0.0: In this case, the aspect ratio is determined from 
;                                 the ratio of yrange to xrange in the first graph.
;                          <0.0: -1.0 x the aspect ratio of the PS file.
;                         Note that the aspect ratio of the graph that is being
;                         determined is that of its box. However, unless xc or yc
;                         are specified, that is also the aspect ratio of the 
;                         graph itself.
;           WAGONCIRCLE: Flag (0/1) or value (0.01-0.50) for wagon circle mode.
;                         It overrides axes styles and graph positions (xc, yc,
;                         xcenter, ycenter). Ignored if FREE is activated.
;           ALIGN:       Graph alignment flag. Ignored if FREE is activated.
;           LANDSCAPE:   Flag for landscape mode.
;           CMYK:        Flag to generate CMYK PS output.
;           TITLE:       Main title.                       
;           TITLEMARGIN: Title margin in normal units (default: 0.05).                       
;           TITLEFONT:   Title font size.                        </PRE>
; @keyword QUIET {in}{optional}{type=boolean}{default=FALSE} Flag to supress informational messages.<BR>
; @keyword VAR {in}{optional}{type=structure} Input structure to pass 
;           additional variables. <BR>
;-
;<PRE>
PRO MULTIGRAPH, psname, $
	            g1,  g2,  g3,  g4,  g5,  g6,  g7,  g8,  g9,  g10, $
                   g11, g12, g13, g14, g15, g16, g17, g18, g19,  g20, $
                   g21, g22, g23, g24, g25, g26, g27, g28, g29,  g30, $
                   g31, g32, g33, g34, g35, g36, g37, g38, g39,  g40, $
                   g41, g42, g43, g44, g45, g46, g47, g48, g49,  g50, $
                   g51, g52, g53, g54, g55, g56, g57, g58, g59,  g60, $
                   g61, g62, g63, g64, g65, g66, g67, g68, g69,  g70, $
                   g71, g72, g73, g74, g75, g76, g77, g78, g79,  g80, $
                   g81, g82, g83, g84, g85, g86, g87, g88, g89,  g90, $
                   g91, g92, g93, g94, g95, g96, g97, g98, g99, g100, $
		   MULTI=multi, QUIET=quiet, VAR=var
ON_ERROR, 1
jmaplot_path = JMAPLOTPATH(JMAPLOT_VERSION=jmaplot_version)
ngraphs = N_PARAMS()-1
IF ngraphs EQ -1 THEN BEGIN
 OPENHTML, jmaplot_path + 'html/multigraph.html'
 RETURN
ENDIF
xsv     = !X
ysv     = !Y
psv     = !P
!X.TICKV[1] = !X.TICKV[0]
!Y.TICKV[1] = !Y.TICKV[0]
!X.RANGE    = [0.0,0.0]
!Y.RANGE    = [0.0,0.0]
IF KEYWORD_SET(VAR)         EQ 0 THEN var         = {dummy:0}
;                                                                     
; Inicializacin de MULTI
;
IF N_ELEMENTS(multi) EQ 0 THEN multi = {dummy:0}
IF SIZE(multi, /TNAME) NE 'STRUCT' THEN MESSAGE, 'MULTI should be a structure'
IF TAG_EXIST(multi,'landscape') THEN $
 COPY_STRUC,  multi, multi, EXCEPT_TAGS='landscape',      STRUC_ADD={  landscape:1*(multi.landscape NE 0)  } ELSE $
 COPY_STRUC,  multi, multi, EXCEPT_TAGS='landscape',      STRUC_ADD={  landscape:0                         }
IF TAG_EXIST(multi,'title') EQ 0 AND TAG_EXIST(multi,'titlemargin') THEN COPY_STRUC, multi, multi, STRUC_ADD={title:''}
IF TAG_EXIST(multi,'title') THEN BEGIN
 IF TAG_EXIST(multi,'titlemargin') THEN $
  COPY_STRUC, multi, multi, EXCEPT_TAGS='titlemargin',    STRUC_ADD={titlemargin:FLOAT(multi.titlemargin)  } ELSE $
  COPY_STRUC, multi, multi, EXCEPT_TAGS='titlemargin',    STRUC_ADD={titlemargin:0.05                      }
 IF multi.titlemargin GT 0.5 THEN MESSAGE, 'Title margin too large'
 IF TAG_EXIST(multi,'titlefont')   THEN $
  COPY_STRUC, multi, multi, EXCEPT_TAGS='titlefont',      STRUC_ADD={  titlefont:FLOAT(multi.titlefont)    } ELSE $
  COPY_STRUC, multi, multi, EXCEPT_TAGS='titlefont',      STRUC_ADD={  titlefont:0.0                       }
ENDIF ELSE BEGIN
 COPY_STRUC,  multi, multi, EXCEPT_TAGS=['titlemargin','titlefont'], $
	                    STRUC_ADD={title:'', titlemargin:0.0, titlefont:0.0}
ENDELSE
IF TAG_EXIST(multi,'free') THEN $
 COPY_STRUC,  multi, multi, EXCEPT_TAGS='free',           STRUC_ADD={       free:1*(multi.free        NE 0)} ELSE $
 COPY_STRUC,  multi, multi, EXCEPT_TAGS='free',           STRUC_ADD={       free:0                         }
IF TAG_EXIST(multi,'wagoncircle') THEN $
 COPY_STRUC,  multi, multi, EXCEPT_TAGS='wagoncircle',    STRUC_ADD={wagoncircle:FLOAT(multi.wagoncircle)  } ELSE $
 COPY_STRUC,  multi, multi, EXCEPT_TAGS='wagoncircle',    STRUC_ADD={wagoncircle:0.0                       }
multi.wagoncircle = multi.wagoncircle*(multi.wagoncircle LT 0.50) + 0.20*(multi.wagoncircle GE 0.50)
IF TAG_EXIST(multi,'align') THEN $
 COPY_STRUC,  multi, multi, EXCEPT_TAGS='align',          STRUC_ADD={      align:1*(multi.align       NE 0)} ELSE $
 COPY_STRUC,  multi, multi, EXCEPT_TAGS='align',          STRUC_ADD={      align:0                         }
COPY_STRUC,   multi, multi, EXCEPT_TAGS=['dummy','type'], STRUC_ADD={       type:INTARR(ngraphs)           }
;                                                                     
; Primer pase de grficas (Comprobacin de tipos)
;
ngraphsr = 0
FOR k=0,ngraphs-1 DO BEGIN
 ks   = STRTRIM(k+1,2)
 gks  = 'g'  + ks
 ggks = 'gg' + ks
 tmp  = EXECUTE('gg=' + gks)
 IF SIZE(gg, /TNAME) NE 'STRUCT' THEN MESSAGE, 'Input is not a structure'
 IF TAG_EXIST(gg,'type') EQ 0 THEN MESSAGE, 'Missing type in ' + gks
 gg.type        = STRLOWCASE(gg.type)
 IF gg.type[0] EQ 'singleplot' OR gg.type EQ 'singlecontour' THEN BEGIN
  multi.type[k] = 1
  ngraphsr      = ngraphsr + 1
 ENDIF
 tmp  = EXECUTE(ggks + '=gg')
ENDFOR
IF ngraphsr EQ 0 THEN MESSAGE, 'You need at least one SINGLEPLOT or SINGLECONTOUR call in MULTIGRAPH'
;                                                                     
; Posicionamiento global 
;
COPY_STRUC, multi, multi, EXCEPT_TAGS='boxes', STRUC_ADD={boxes:FLTARR(4,ngraphs)}
IF multi.free EQ 0 THEN BEGIN ; Modo automtico (con malla)
 IF TAG_EXIST(multi,'xyn') THEN BEGIN     ; Eleccin de malla manual
  IF N_ELEMENTS(multi.xyn) NE 2 THEN MESSAGE, 'xyn has to be a two-element array'
  COPY_STRUC, multi, multi, EXCEPT_TAGS='xyn', STRUC_ADD={xyn:REFORM(FIX(multi.xyn))}
  IF multi.xyn[0]*multi.xyn[1] LT ngraphs THEN MESSAGE, 'xyn too small'
 ENDIF ELSE BEGIN                         ; Eleccin de malla automtica
  FOR ngx=1,FIX(SQRT(1.0*ngraphs)) DO BEGIN
   ngy = FLOAT(ngraphs)/ngx
   IF FIX(ngy) EQ ngy THEN COPY_STRUC, multi, multi, EXCEPT_TAGS='xyn', STRUC_ADD={xyn:[ngx,FIX(ngy)]}
  ENDFOR
  IF multi.landscape EQ 1 THEN multi.xyn = REVERSE(multi.xyn)
 ENDELSE
 itoij = INTARR(ngraphs,2)
 ijtoi = INTARR(multi.xyn) -1
 FOR i=0,multi.xyn[0]-1 DO BEGIN
  FOR j=0,multi.xyn[1]-1 DO BEGIN
   IF i+j*multi.xyn[0] LT ngraphs THEN BEGIN
    itoij[i+j*multi.xyn[0],*] = [i,j]
    ijtoi[i               ,j] = i+j*multi.xyn[0]
   ENDIF
  ENDFOR
 ENDFOR
 IF multi.wagoncircle EQ 0 THEN BEGIN ; Caso normal
;  mx = 0.01*multi.xyn[0]/(multi.xyn[0]+0.1)
;  my = 0.01*multi.xyn[1]/(multi.xyn[1]+0.1)
  mx = 0.00
  my = 0.00
  multi.boxes[0,*] = (itoij[*,0]   +mx)/multi.xyn[0]
  multi.boxes[1,*] = (itoij[*,1]   +my)/multi.xyn[1]*(1.-multi.titlemargin)
  multi.boxes[2,*] = (itoij[*,0]+1.-mx)/multi.xyn[0]
  multi.boxes[3,*] = (itoij[*,1]+1.-my)/multi.xyn[1]*(1.-multi.titlemargin)
 ENDIF ELSE BEGIN                     ; Defendindose de los indios
  xc      = FLTARR(multi.xyn) ; De momento, en coordenadas relativas
  yc      = FLTARR(multi.xyn)
  xsize   = FLTARR(multi.xyn)
  ysize   = FLTARR(multi.xyn)
  xlstyle = INTARR(multi.xyn)
  ylstyle = INTARR(multi.xyn)
  xustyle = INTARR(multi.xyn)
  yustyle = INTARR(multi.xyn)
  epsx    = 0.010
  epsy    = 0.010
  IF multi.xyn[0] EQ 1 THEN BEGIN  ; Columna nica
   xc[*,*]                      = 0.50
   xsize[*,*]                   = 1.00 - 2.0*multi.wagoncircle
   ylstyle[*,*]                 = 1
   yustyle[*,*]                 = 1
  ENDIF ELSE BEGIN                 ; Por lo menos dos columnas
   xc[0,*]                      = 0.50 + 0.5*(multi.wagoncircle - epsx)
   xsize[0,*]                   = 1.00 - epsx - multi.wagoncircle
   ylstyle[0,*]                 = 1
   yustyle[0,*]                 = 0
   xc[multi.xyn[0]-1,*]         = 0.50 - 0.5*(multi.wagoncircle - epsx)
   xsize[multi.xyn[0]-1,*]      = 1.00 - epsx - multi.wagoncircle
   ylstyle[multi.xyn[0]-1,*]    = 0
   yustyle[multi.xyn[0]-1,*]    = 1
   IF multi.xyn[0] GE 3 THEN BEGIN ; Por lo menos tres columnas
    xc[1:multi.xyn[0]-2,*]      = 0.50
    xsize[1:multi.xyn[0]-2,*]   = 1.00 - 2.0*epsx
    ylstyle[1:multi.xyn[0]-2,*] = 0
    yustyle[1:multi.xyn[0]-2,*] = 0
   ENDIF
  ENDELSE
  IF multi.xyn[1] EQ 1 THEN BEGIN  ; Fila nica
   yc[*,*]                      = 0.50
   ysize[*,*]                   = 1.00 - 2.0*multi.wagoncircle
   xlstyle[*,*]                 = 1
   xustyle[*,*]                 = 1
  ENDIF ELSE BEGIN                 ; Por lo menos dos filas
   yc[*,0]                      = 0.50 + 0.5*(multi.wagoncircle - epsy)
   ysize[*,0]                   = 1.00 - epsy - multi.wagoncircle
   xlstyle[*,0]                 = 1
   xustyle[*,0]                 = 0
   yc[*,multi.xyn[1]-1]         = 0.50 - 0.5*(multi.wagoncircle - epsy)
   ysize[*,multi.xyn[1]-1]      = 1.00 - epsy - multi.wagoncircle
   xlstyle[*,multi.xyn[1]-1]    = 0
   xustyle[*,multi.xyn[1]-1]    = 1
   IF multi.xyn[1] GE 3 THEN BEGIN ; Por lo menos tres filas
    yc[*,1:multi.xyn[1]-2]      = 0.50
    ysize[*,1:multi.xyn[1]-2]   = 1.00 - 2*epsy
    xlstyle[*,1:multi.xyn[1]-2] = 0
    xustyle[*,1:multi.xyn[1]-2] = 0
   ENDIF
  ENDELSE
  suminvx = TOTAL(1./xsize[*,0])
  suminvy = TOTAL(1./ysize[0,*])
  FOR k=0,ngraphs-1 DO BEGIN
   IF itoij[k,0] EQ 0 THEN multi.boxes[0,k] = 0.0 ELSE multi.boxes[0,k] = multi.boxes[2,ijtoi[itoij[k,0]-1,itoij[k,1]  ]]
   IF itoij[k,1] EQ 0 THEN multi.boxes[1,k] = 0.0 ELSE multi.boxes[1,k] = multi.boxes[3,ijtoi[itoij[k,0]  ,itoij[k,1]-1]]
   multi.boxes[2,k] = multi.boxes[0,k] +                     1./(xsize[itoij[k,0],itoij[k,1]]*suminvx)
   multi.boxes[3,k] = multi.boxes[1,k] + (1.-multi.titlemargin)/(ysize[itoij[k,0],itoij[k,1]]*suminvy)
   IF itoij[k,0] EQ multi.xyn[0]-1 AND ABS(1.                  -multi.boxes[2,k]) GT 1.e-3 THEN MESSAGE, 'Wagon-circling algorithm x failure'
   IF itoij[k,1] EQ multi.xyn[1]-1 AND ABS(1.-multi.titlemargin-multi.boxes[3,k]) GT 1.e-3 THEN MESSAGE, 'Wagon-circling algorithm y failure'
   ks   = STRTRIM(k+1,2)
   ggks = 'gg' + ks
   tmp  = EXECUTE('gg=' + ggks)
   IF multi.type[k] EQ 1 THEN BEGIN
    ppp = {   xc:multi.boxes[0,k] + xc[itoij[k,0],itoij[k,1]]*(multi.boxes[2,k]-multi.boxes[0,k]), $  
              yc:multi.boxes[1,k] + yc[itoij[k,0],itoij[k,1]]*(multi.boxes[3,k]-multi.boxes[1,k]), $
	   xsize:                xsize[itoij[k,0],itoij[k,1]]*(multi.boxes[2,k]-multi.boxes[0,k]), $
	   ysize:                ysize[itoij[k,0],itoij[k,1]]*(multi.boxes[3,k]-multi.boxes[1,k])}
    IF TAG_EXIST(gg,'plot') THEN BEGIN
     pp = gg.plot
     COPY_STRUC, pp, pp, EXCEPT_TAGS=['xc','yc','xsize','ysize'], STRUC_ADD=ppp
     COPY_STRUC, gg, gg, EXCEPT_TAGS='plot', STRUC_ADD={plot:pp}
    ENDIF ELSE BEGIN
     COPY_STRUC, gg, gg, EXCEPT_TAGS='plot', STRUC_ADD={plot:ppp}
    ENDELSE
    IF TAG_EXIST(gg,'xaxisl') THEN BEGIN
     xx = gg.xaxisl
     COPY_STRUC, xx, xx, EXCEPT_TAGS=['style'], STRUC_ADD={        style:xlstyle[itoij[k,0],itoij[k,1]]}
     COPY_STRUC, gg, gg, EXCEPT_TAGS='xaxisl',  STRUC_ADD={xaxisl:xx}
    ENDIF ELSE BEGIN
     COPY_STRUC, gg, gg, EXCEPT_TAGS='xaxisl',  STRUC_ADD={xaxisl:{style:xlstyle[itoij[k,0],itoij[k,1]]}}
    ENDELSE
    IF TAG_EXIST(gg,'yaxisl') THEN BEGIN
     yy = gg.yaxisl
     COPY_STRUC, yy, yy, EXCEPT_TAGS=['style'], STRUC_ADD={        style:ylstyle[itoij[k,0],itoij[k,1]]}
     COPY_STRUC, gg, gg, EXCEPT_TAGS='yaxisl',  STRUC_ADD={yaxisl:yy}
    ENDIF ELSE BEGIN
     COPY_STRUC, gg, gg, EXCEPT_TAGS='yaxisl',  STRUC_ADD={yaxisl:{style:ylstyle[itoij[k,0],itoij[k,1]]}}
    ENDELSE
    IF TAG_EXIST(gg,'xaxisu') THEN BEGIN
     xx = gg.xaxisu
     COPY_STRUC, xx, xx, EXCEPT_TAGS=['style'], STRUC_ADD={        style:xustyle[itoij[k,0],itoij[k,1]]}
     COPY_STRUC, gg, gg, EXCEPT_TAGS='xaxisu',  STRUC_ADD={xaxisu:xx}
    ENDIF ELSE BEGIN
     COPY_STRUC, gg, gg, EXCEPT_TAGS='xaxisu',  STRUC_ADD={xaxisu:{style:xustyle[itoij[k,0],itoij[k,1]]}}
    ENDELSE
    IF TAG_EXIST(gg,'yaxisu') THEN BEGIN
     yy = gg.yaxisu
     COPY_STRUC, yy, yy, EXCEPT_TAGS=['style'], STRUC_ADD={        style:yustyle[itoij[k,0],itoij[k,1]]}
     COPY_STRUC, gg, gg, EXCEPT_TAGS='yaxisu',  STRUC_ADD={yaxisu:yy}
    ENDIF ELSE BEGIN
     COPY_STRUC, gg, gg, EXCEPT_TAGS='yaxisu',  STRUC_ADD={yaxisu:{style:yustyle[itoij[k,0],itoij[k,1]]}}
    ENDELSE
    tmp  = EXECUTE(ggks + '=gg')
   ENDIF
  ENDFOR
 ENDELSE
ENDIF ELSE BEGIN                         ; Modo manual (sin malla)
 COPY_STRUC, multi, multi, EXCEPT_TAGS=['xyn'], STRUC_ADD={xyn:[1,1]}
 mx = 0.00
 my = 0.00
 multi.boxes[0,*] =     mx
 multi.boxes[1,*] =     my /(1.-multi.titlemargin)
 multi.boxes[2,*] = (1.-mx)
 multi.boxes[3,*] = (1.-my)/(1.-multi.titlemargin)
 FOR k=0,ngraphs-1 DO BEGIN
  ks   = STRTRIM(k+1,2)
  ggks = 'gg' + ks
  tmp  = EXECUTE('gg=' + ggks)
  IF multi.type[k] EQ 1 THEN BEGIN
   IF TAG_EXIST(gg.plot,'xc')    EQ 0 OR TAG_EXIST(gg.plot,'yc')    EQ 0 OR $
      TAG_EXIST(gg.plot,'xsize') EQ 0 OR TAG_EXIST(gg.plot,'ysize') EQ 0 THEN $
      MESSAGE, 'If free is activated, you need to specify xc, yc, xsize, and ysize'
  ENDIF
 ENDFOR
ENDELSE
;                                                                     
; Segundo pase de grficas (Inicializacin, datos y ejes)
;
FOR k=0,ngraphs-1 DO BEGIN
 ks   = STRTRIM(k+1,2)
 ggks = 'gg' + ks
 tmp  = EXECUTE('gg=' + ggks)
 IF multi.type[k] EQ 1 THEN BEGIN
  tmp           = EXECUTE(gg.type + ", '', QUIET=quiet, VAR=var, IN=gg, OUT=ggg, EXIT=1")
  gg            = CREATE_STRUCT(ggg, {type:gg.type})
  tmp           = EXECUTE(ggks + '=gg')
 ENDIF
ENDFOR
;                                                                     
; Razn de aspecto y apertura del fichero PS
;
IF TAG_EXIST(multi,'aspect') EQ 0 THEN BEGIN ; Modo automtico: Cada grfica va en un cuadrado
 gaspect   = 1.0
 psaspect  = multi.xyn[1]/(multi.xyn[0]*(1.-multi.titlemargin))*gaspect
 multi     = CREATE_STRUCT(multi, {aspect:gaspect})
ENDIF ELSE BEGIN                             ; Modo manual
 IF multi.aspect GT 0 THEN BEGIN             ; Opcin (a): Se fija el aspecto de las grficas
  gaspect  = multi.aspect
  psaspect = multi.xyn[1]/(multi.xyn[0]*(1.-multi.titlemargin))*gaspect
 ENDIF ELSE IF multi.aspect LT 0 THEN BEGIN  ; Opcin (b): Se fija el aspecto del fichero PS
  psaspect = ABS(multi.aspect)
  gaspect  =              (multi.xyn[0]*(1.-multi.titlemargin))*psaspect/multi.xyn[1]
 ENDIF ELSE BEGIN                            ; Opcin (c): Se fija el aspecto a partir de la primera grfica
  IF multi.type[0] EQ 0 THEN MESSAGE, 'You cannot set aspect=0 when the first graph is not SINGLEPLOT or SINGLECONTOUR'
  gaspect  = gg1.yaxisl.delta/gg1.xaxisl.delta
  psaspect = multi.xyn[1]/(multi.xyn[0]*(1.-multi.titlemargin))*gaspect
 ENDELSE
ENDELSE
COPY_STRUC,  multi, multi, EXCEPT_TAGS=['gaspect','psaspect'], STRUC_ADD={gaspect:gaspect, psaspect:psaspect}
PREPPS, psname, multi, dummy, QUIET=quiet
;                                                                     
; Tercer pase de grficas (Posicionamiento y color)
;
FOR k=0,ngraphs-1 DO BEGIN
 IF multi.type[k] EQ 1 THEN BEGIN
  ks       = STRTRIM(k+1,2)
  ggks     = 'gg' + ks
  tmp      = EXECUTE('gg=' + ggks)
  pp       = gg.plot
  IF multi.free EQ 0 OR TAG_EXIST(pp,'box') EQ 0 THEN $
   COPY_STRUC, pp, pp, EXCEPT_TAGS=['box','psxsize','psysize','psaspect','psxoffset','psyoffset','portrait','landscape'],   $
               STRUC_ADD={box:REFORM(multi.boxes[*,k]),psxsize:multi.psxsize,psysize:multi.psysize,psaspect:multi.psaspect, $
	                  psxoffset:multi.psxoffset,psyoffset:multi.psyoffset,portrait:multi.portrait,landscape:multi.landscape} ELSE $
   COPY_STRUC, pp, pp, EXCEPT_TAGS=['psxsize','psysize','psaspect','psxoffset','psyoffset','portrait','landscape'],   $
               STRUC_ADD={psxsize:multi.psxsize,psysize:multi.psysize,psaspect:multi.psaspect, $
	                  psxoffset:multi.psxoffset,psyoffset:multi.psyoffset,portrait:multi.portrait,landscape:multi.landscape}
  COPY_STRUC, gg, gg, EXCEPT_TAGS='plot', STRUC_ADD={plot:pp}
  tmp      = EXECUTE(gg.type + ", '', QUIET=quiet, VAR=var, IN=gg, OUT=ggg, ENTRANCE=1, EXIT=2")
  gg       = CREATE_STRUCT(ggg, {type:gg.type})
  tmp      = EXECUTE(ggks + '=gg')
 ENDIF
ENDFOR
;                                                                     
; Alineamiento
;
IF multi.free EQ 0 AND multi.align EQ 1 THEN BEGIN
 xsize   = FLTARR(multi.xyn[0])
 xc      = FLTARR(multi.xyn[0])
 extraxl = FLTARR(multi.xyn[0])
 extraxu = FLTARR(multi.xyn[0])
 FOR i=0,multi.xyn[0]-1 DO BEGIN ; Alineamiento horizontal (primer paso)
  pos = WHERE(itoij[*,0] EQ i AND multi.type EQ 1, npos)
  IF npos EQ 0 THEN MESSAGE, 'Column ' + STRTRIM(i+1,2) + ' cannot be aligned for lack of SINGLEPLOT and SINGLECONTOUR'
  tmp = EXECUTE('extraxlt = [gg' + STRTRIM(pos[0]+1,2) + '.plot.extraxl]')
  tmp = EXECUTE('extraxut = [gg' + STRTRIM(pos[0]+1,2) + '.plot.extraxu]')
  FOR k=1,npos-1 DO tmp = EXECUTE('extraxlt = [extraxlt, gg' + STRTRIM(pos[k]+1,2) + '.plot.extraxl]')
  FOR k=1,npos-1 DO tmp = EXECUTE('extraxut = [extraxut, gg' + STRTRIM(pos[k]+1,2) + '.plot.extraxu]')
  extraxl[i] = MAX(extraxlt)
  extraxu[i] = MAX(extraxut)
  xsize[i]   = (multi.boxes[2,ijtoi[i,0]]-multi.boxes[0,ijtoi[i,0]]) - (extraxl[i]+extraxu[i])
  IF xsize[i] LE 0.0 THEN MESSAGE, 'Column ' + STRTRIM(i+1,2) + ' cannot be aligned because axes text is too large'
  xc[i]      = multi.boxes[0,ijtoi[i,0]] + extraxl[i] + 0.5*xsize[i]
  FOR k=0,npos-1 DO tmp = EXECUTE('gg' + STRTRIM(pos[k]+1,2) + '.plot.xsize = xsize[i]')
  FOR k=0,npos-1 DO tmp = EXECUTE('gg' + STRTRIM(pos[k]+1,2) + '.plot.xc    = xc[i]   ')
 ENDFOR
 ysize   = FLTARR(multi.xyn[1])
 yc      = FLTARR(multi.xyn[1])
 extrayl = FLTARR(multi.xyn[1])
 extrayu = FLTARR(multi.xyn[1])
 FOR j=0,multi.xyn[1]-1 DO BEGIN ; Alineamiento vertical (primer paso)
  pos = WHERE(itoij[*,1] EQ j AND multi.type EQ 1, npos)
  IF npos EQ 0 THEN MESSAGE, 'Row '    + STRTRIM(j+1,2) + ' cannot be aligned for lack of SINGLEPLOT and SINGLECONTOUR'
  tmp = EXECUTE('extraylt = [gg' + STRTRIM(pos[0]+1,2) + '.plot.extrayl]')
  tmp = EXECUTE('extrayut = [gg' + STRTRIM(pos[0]+1,2) + '.plot.extrayu]')
  FOR k=1,npos-1 DO tmp = EXECUTE('extraylt = [extraylt, gg' + STRTRIM(pos[k]+1,2) + '.plot.extrayl]')
  FOR k=1,npos-1 DO tmp = EXECUTE('extrayut = [extrayut, gg' + STRTRIM(pos[k]+1,2) + '.plot.extrayu]')
  extrayl[j] = MAX(extraylt)
  extrayu[j] = MAX(extrayut)
  ysize[j]   = (multi.boxes[3,ijtoi[0,j]]-multi.boxes[1,ijtoi[0,j]]) - (extrayl[j]+extrayu[j])
  IF ysize[j] LE 0.0 THEN MESSAGE, 'Column ' + STRTRIM(j+1,2) + ' cannot be aligned because axes text is too large'
  yc[j]      = multi.boxes[1,ijtoi[0,j]] + extrayl[j] + 0.5*ysize[j]
  FOR k=0,npos-1 DO tmp = EXECUTE('gg' + STRTRIM(pos[k]+1,2) + '.plot.ysize = ysize[j]')
  FOR k=0,npos-1 DO tmp = EXECUTE('gg' + STRTRIM(pos[k]+1,2) + '.plot.yc    = yc[j]   ')
 ENDFOR
 IF multi.aspect GE 0 THEN BEGIN ; Si el aspecto no se refiere al fichero PS hay que realinear (todas las grficas del mismo tamao)
  extraxl    = MAX(extraxl)
  extraxu    = MAX(extraxu)
  extrayl    = MAX(extrayl)
  extrayu    = MAX(extrayu)
  xsize      = (multi.boxes[2,0]-multi.boxes[0,0]) - (extraxl+extraxu)
  ysize      = (multi.boxes[3,0]-multi.boxes[1,0]) - (extrayl+extrayu)
  IF xsize LE 0.0 OR ysize LE 0.0 THEN MESSAGE, 'ALIGNMENT failed because axes text is too large'
  pos = WHERE(multi.type EQ 1,npos)
  FOR k=0,npos-1 DO tmp = EXECUTE('gg' + STRTRIM(pos[k]+1,2) + '.plot.xsize = xsize')
  FOR k=0,npos-1 DO tmp = EXECUTE('gg' + STRTRIM(pos[k]+1,2) + '.plot.ysize = ysize')
  FOR k=0,npos-1 DO tmp = EXECUTE('gg' + STRTRIM(pos[k]+1,2) + '.plot.xc    = multi.boxes[0,pos[k]] + extraxl + 0.5*xsize')
  FOR k=0,npos-1 DO tmp = EXECUTE('gg' + STRTRIM(pos[k]+1,2) + '.plot.yc    = multi.boxes[1,pos[k]] + extrayl + 0.5*ysize')
 ENDIF
 FOR k=0,ngraphs-1 DO BEGIN ; Ejecutamos GRPOS
  IF multi.type[k] EQ 1 THEN BEGIN
   ks            = STRTRIM(k+1,2)
   ggks          = 'gg' + ks
   tmp           = EXECUTE('gg=' + ggks)
   pp            = gg.plot
   GRPOS, pp, gg.xaxisl, gg.yaxisl, gg.xaxisu, gg.yaxisu
   COPY_STRUC, gg, gg, EXCEPT_TAGS='plot', STRUC_ADD={plot:pp}
   tmp            = EXECUTE(ggks + '=gg')
  ENDIF
 ENDFOR
ENDIF
;                                                                     
; Cuarto pase de grficas (Impresin)
;
charsize   = 0.0
FOR k=0,ngraphs-1 DO BEGIN
 ks        = STRTRIM(k+1,2)
 ggks      = 'gg' + ks
 tmp       = EXECUTE('gg=' + ggks)
 IF multi.type[k] EQ 1 THEN BEGIN
  tmp      = EXECUTE(gg.type + ", '', QUIET=quiet, VAR=var, IN=gg, OUT=ggg, ENTRANCE=2")
  charsize = charsize*(gg.plot.charsize LT charsize) + gg.plot.charsize*(gg.plot.charsize GE charsize)
  font     = gg.plot.font
  def      = gg.cfile.def
 ENDIF ELSE BEGIN
  tmp      = EXECUTE(gg.type)
 ENDELSE
ENDFOR
IF multi.title NE '' THEN BEGIN
 titlen = LENGTHV2(multi.title)
 IF multi.titlefont EQ 0 THEN $
  multi.titlefont = 3.0*MIN([multi.psxsize/titlen,0.7*multi.titlemargin*multi.psysize])
 XYOUTS, 0.5, 1.0-0.9*multi.titlemargin, multi.title, $
         FONT=font, CHARSIZE=multi.titlefont, ALIGNMENT=0.5, /NORMAL, COLOR=def
ENDIF
;
; Cierre del fichero PostScript
;
CLOSEPS
!X     = xsv
!Y     = ysv
!P     = psv
END
;</PRE>
