;+
; READDATA opens a data file and reads up to 82 columns. No header information 
;  is needed and lines that do not adhere to the expected format are
;  interpreted as comments. 
; See the <a HREF="../readdata.pro">full procedure</a><BR>
; @examples <PRE>
;  READDATA can be used as: 
;    READDATA, 'test.dat', a, b, c, d                              ; Unformatted data (double precision)
;    READDATA, 'test.dat', x, y, z, FORMAT='(F,I,A)'               ; Simplified format data
;    READDATA, 'test.dat', a, b, c, d, FORMAT='(2(D16.6,X),I5,A6)' ; Full format data
;    READDATA, 'test.dat', x, /ONESTRING                           ; Each line is returned as a string 
;    READDATA, 'test.dat', x, /STRUCTURE                           ; A structure is used as return </PRE>
; @version v. 3.1, 11 May 2013.                         <BR>
; @history <PRE>
;
;  v0.0: First version.   
;  v0.1: Strings allowed.
;  v1.0: General cleanup.
;  v1.1: FORMAT and SKIPCH introduced.
;        Name changed from READDATA2 to READDATA.
;        A bug with ncompi is fixed.
;  v1.2: General revision based oon READCOL.                               
;        NMAX is eliminated.
;        Two types of format (full and simplified) are allowed.
;        SEPCH introduced.
;        QUIET introduced.
;  v1.3: New format 'E'.
;  v1.4: Number of columns increased from 50 to 80.
;  v2.0: English version.
;  v2.1: IDLdoc documentation.
;  v2.2: OPENHTML introduced.
;        Upgrade to IDLdoc documentation. 
;  v2.3: Number of columns increased from 80 to 82.
;  v2.4: Bug with NP corrected. 
;        Bug with L format fixed.
;        Bug caused by update in astronomy library fixed.
;        STRNUMBER substituted by VALID_NUM.
;  v2.5: Version system update.  
;        TEXT introduced.                
;        Format issues with new IDL versions solved, but note the strange behavior that
;         IDL has now (has ever?): READS, '2.3', v1, FORMAT='(I3)' gives v1 = 2 and      
;         READS, '234', v1, FORMAT='(F3.1)' gives v1 = 234.000 i.e. IDL does not really
;         care about decimal point placement in this respect! 
;  v3.0: Adapted to SINGLEPLOT and SINGLECONTOUR changes.
;        HTML pages moved to local directory.
;  v3.1: Tabs converted to spaces in input. 
;        STRTRIM added.
;        STRUCTURE, TAGS added. </PRE>
; @author Jess Maz Apellniz, IAA   <BR>
; @param   file {in}{required}{type=string} File name.                                                        <BR>
; @param   v1 {in}{required} First variable to be read.                                        <BR>
; @param   v2 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v3 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v4 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v5 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v6 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v7 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v8 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v9 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v10 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v11 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v12 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v13 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v14 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v15 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v16 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v17 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v18 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v19 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v20 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v21 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v22 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v23 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v24 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v25 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v26 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v27 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v28 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v29 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v30 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v31 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v32 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v33 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v34 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v35 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v36 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v37 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v38 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v39 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v40 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v41 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v42 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v43 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v44 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v45 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v46 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v47 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v48 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v49 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v50 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v51 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v52 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v53 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v54 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v55 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v56 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v57 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v58 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v59 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v60 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v61 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v62 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v63 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v64 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v65 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v66 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v67 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v68 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v69 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v70 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v71 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v72 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v73 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v74 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v75 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v76 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v77 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v78 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v79 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v80 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v81 {in}{optional} Additional variable to be read.                                        <BR>
; @param   v82 {in}{optional} Additional variable to be read.                                        <BR>
; @keyword NP{out}{optional}{type=integer} Number of elements in each variable.<BR>
; @keyword ERR{out}{optional}{type=integer} Error flag.                                              <BR>
; @keyword FORMAT{in}{optional}{type=string} Format of the (simplified) type 
;           '(F,I,A)' or of the (full) type '(3(X,A2),I2,F5.1)'. Variable types 
;           allowed: A, B, D, E, F, I, L, and X (the latter, only full format).    <BR>
; @keyword SKIPCH {in}{optional}{type=string} Character to skip a line (use it 
;           to begin those lines).           <BR>
; @keyword SEPCH{in}{optional}{type=string}{default=" "} Separator to use for 
;           unformatted or simple-format cases.<BR>
; @keyword STRTRIM{in}{optional}{type=boolean}{default=FALSE} Flag to eliminate
;           white spaces with simple format when SEPCH is not " ".
; @keyword ONESTRING{in}{optional}{type=boolean}{default=FALSE} Flag to read 
;           each line as a single string.                        <BR>
; @keyword STRUCTURE{in}{optional}{type=integer}{default=0} Flag to read 
;           each line as a structure. It also selects the line number to read the tags. <BR>
; @keyword TAGS{out}{optional}{type=string} Tag names (see structure). <BR>
; @keyword TEXT{out}{optional}{type=string} Lines that do not conform to the format. <BR>
; @keyword QUIET{in}{optional}{type=boolean}{default=FALSE} Flag to supress informational messages. <BR>
;-
;<PRE>
PRO READDATA, file,  v1,  v2,  v3,  v4,  v5,  v6,  v7,  v8,  v9, v10, $
                    v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, $
                    v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, $
                    v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, $
                    v41, v42, v43, v44, v45, v46, v47, v48, v49, v50, $
                    v51, v52, v53, v54, v55, v56, v57, v58, v59, v60, $
                    v61, v62, v63, v64, v65, v66, v67, v68, v69, v70, $
                    v71, v72, v73, v74, v75, v76, v77, v78, v79, v80, $
                    v81, v82, $
                    NP=np, ERR=err, FORMAT=format, SKIPCH=skipch, SEPCH=sepch, STRTRIM=strtrim, $
                    ONESTRING=onestring, STRUCTURE=structure, TAGS=tags, TEXT=text, QUIET=quiet
ON_ERROR, 2
IF N_PARAMS() EQ 0 THEN BEGIN
 jmaplot_path = JMAPLOTPATH(JMAPLOT_VERSION=jmaplot_version)
 OPENHTML, jmaplot_path + 'html/readdata.html'
 RETURN
ENDIF
IF KEYWORD_SET(QUIET)     EQ 0 THEN quiet     = 0
IF KEYWORD_SET(STRUCTURE) EQ 0 THEN structure = 0
flag = 0
;
; Comprobacin de parmetros
;
nvar = N_PARAMS() - 1
OPENR, unit, file, /GET_LUN, ERR=err
IF err NE 0 THEN BEGIN
 IF quiet LT 2 THEN PRINT, 'READDATA failed, ' + file + ' has no lines'
 err = 1
 RETURN
ENDIF
FREE_LUN,unit
nmax = FILE_LINES(file)
IF KEYWORD_SET(SKIPCH) EQ 0 THEN skipch = ""
IF ARG_PRESENT(V1) EQ 0 THEN BEGIN
 IF quiet LT 2 THEN PRINT, 'READDATA failed, at least one vector is needed'
 err = 1
 RETURN
ENDIF
since_v53 = !VERSION.RELEASE GE '5.3'
IF KEYWORD_SET(SEPCH) EQ 0 THEN sepch = ' ' 
IF KEYWORD_SET(STRUCTURE) AND KEYWORD_SET(FORMAT) EQ 0 THEN BEGIN
 IF quiet LT 2 THEN PRINT, 'READDATA failed, FORMAT is needed for structures'
 err = 1
 RETURN
ENDIF
vv = 'v' + STRTRIM(INDGEN(nvar)+1,2)
;
; Procesado del formateado (pasos iniciales)
;
idltype = INTARR(nvar)
IF KEYWORD_SET(FORMAT) NE 0 THEN BEGIN
 format  = STRUPCASE(format)
 format2 = format
 IF STRMID(format2,0,1)                EQ '(' THEN $
    format2 = STRMID(format2,1,9999)
 IF STRMID(format2,STRLEN(format2)-1,1) EQ ')' THEN $
    format2 = STRMID(format2,0,STRLEN(format2)-1)
 pos1 = STRALLPOS(format2,'(')
 pos2 = STRALLPOS(format2,')')
 format3 = format2
;
; Expansin de parntesis
;
 IF pos1[0] NE -1 OR pos2[0] NE -1 THEN BEGIN 
  IF (pos1[0] EQ -1 AND pos2[0] NE -1) OR (pos1[0] NE -1 AND pos2[0] EQ -1) $
    OR N_ELEMENTS(pos1) NE N_ELEMENTS(pos2) THEN BEGIN
   IF quiet LT 2 THEN PRINT, 'READDATA failed, incorrect FORMAT'
   err = 1
   RETURN
  ENDIF
  nk      = N_ELEMENTS(pos2)
  FOR k=0,nk-1 DO BEGIN
   pos1  = STRALLPOS(format3,'(')
   pos2  = STRALLPOS(format3,')')
   pos20 = pos2[0]
   aux   = (pos2[0]-pos1)*(pos1 LT pos2[0]) + 9999*(pos1 GT pos2[0])
   aux2  = MIN(aux,aux3)
   pos10 = pos1[aux3]
;   ch1   = STRNUMBER(STRMID(format3,pos10-1,1), un)
   ch1   = VALID_NUM(STRMID(format3,pos10-1,1), un)
   IF pos10 EQ 1 THEN BEGIN
    dc  = 0
    ch2 = 1
   ENDIF ELSE BEGIN
    IF STRMID(format3,pos10-2,1) EQ ',' OR $
       STRMID(format3,pos10-2,1) EQ '(' THEN BEGIN
     dc  = 0
     ch2 = 1
    ENDIF ELSE BEGIN
;     ch2 = STRNUMBER(STRMID(format3,pos10-2,1), dc)
     ch2 = VALID_NUM(STRMID(format3,pos10-2,1), dc)
    ENDELSE
   ENDELSE
   IF ch1 EQ 0 OR ch2 EQ 0 THEN BEGIN
    IF quiet LT 2 THEN PRINT, 'READDATA failed, incorrect FORMAT'
    err = 1
    RETURN
   ENDIF
   num = 10*dc + un
   IF dc EQ 0 THEN formatn = STRMID(format3,0,pos10-1) ELSE $
                   formatn = STRMID(format3,0,pos10-2)
   FOR l=1,num DO formatn = formatn + $
                            STRMID(format3,pos10+1,pos20-pos10-1) + ','
   IF pos20 EQ STRLEN(format3)-1 THEN BEGIN 
    formatn = STRMID(formatn,0,STRLEN(formatn)-1)
   ENDIF ELSE BEGIN
    IF STRMID(format3,pos20+1,1) EQ ')'                THEN $
      formatn = STRMID(formatn,0,STRLEN(formatn)-1) +       $
                STRMID(format3,pos20+1,9999)           ELSE $
      formatn = formatn + STRMID(format3,pos20+2,9999) 
   ENDELSE
   format3 = formatn
  ENDFOR
 ENDIF
;
; Desarrollo de componentes mltiples
;
 aux = STRALLPOS(format3,',')
 IF aux[0] EQ -1 THEN nf = 1 ELSE nf = N_ELEMENTS(aux)+1
 aux     = format3
 format4 = ''
 FOR k=0,nf-1 DO BEGIN
  aux2  = GETTOK(aux,',') 
  vtype = STRMID(aux2,0,1)
  num = 1
;  IF STRNUMBER(vtype,val) EQ 1 THEN BEGIN
  IF VALID_NUM(vtype,val) EQ 1 THEN BEGIN
   num   = val
   vtype = STRMID(aux2,1,1)
;   IF STRNUMBER(vtype,val) EQ 1 THEN BEGIN
   IF VALID_NUM(vtype,val) EQ 1 THEN BEGIN
    num   = 10*num+ val
    vtype = STRMID(aux2,2,1)
;    IF STRNUMBER(vtype,val) EQ 1 THEN BEGIN
    IF VALID_NUM(vtype,val) EQ 1 THEN BEGIN
     num   = 10*num+ val
     FOR l=1,num DO format4 = format4 + STRMID(aux2,3,9999) + ','
    ENDIF ELSE BEGIN
     FOR l=1,num DO format4 = format4 + STRMID(aux2,2,9999) + ','
    ENDELSE
   ENDIF ELSE BEGIN
    FOR l=1,num DO format4 = format4 + STRMID(aux2,1,9999) + ','
   ENDELSE
  ENDIF ELSE BEGIN
   format4 = format4 + aux2 + ','
  ENDELSE
 ENDFOR
 format4 = STRMID(format4,0,STRLEN(format4)-1)
 formate = '(' + format4 + ')'
 FOR i=0,STRLEN(formate)-1 DO IF STRMID(formate,i,1) EQ 'L' THEN STRPUT, formate, 'I', i
;
; Procesado del formateado (pasos finales)
;
 format5 = format4
 chars   = ['0','1','2','3','4','5','6','7','8','9']
 FOR i=0,N_ELEMENTS(chars)-1 DO REMCHAR, format5, chars[i]
 IF format4 EQ format5 THEN fullf = 0 ELSE fullf = 1
 chars   = ['X',',','.',' ']
 FOR i=0,N_ELEMENTS(chars)-1 DO REMCHAR, format5, chars[i]
 format6 = format5
 chars   = ['A','D','F','E','I','B','L']
 FOR i=0,N_ELEMENTS(chars)-1 DO REMCHAR, format6, chars[i]
 IF format6 NE '' THEN BEGIN
  IF quiet LT 2 THEN PRINT, 'READDATA failed, illegal characters in FORMAT: ', format6
  err = 1
  RETURN
 ENDIF
 IF KEYWORD_SET(STRUCTURE) THEN BEGIN
  AUTOFORMAT:
  nvar    = STRLEN(format5)
  vv      = 'v' + STRTRIM(INDGEN(nvar)+1,2)
  tags    = vv
  idltype = INTARR(nvar)
 ENDIF 
 IF nvar NE STRLEN(format5) THEN BEGIN
  IF quiet LT 2 THEN PRINT, nvar, STRLEN(format5)
  IF quiet LT 2 THEN PRINT, 'READDATA failed, number of variables (' + STRTRIM(STRLEN(format5),2) + $
	                    ') in FORMAT incorrect (' + STRTRIM(nvar,2) + ' expected)'
  err = 1
  RETURN
 ENDIF
 IF fullf EQ 1 THEN BEGIN      ; Para el formato pleno convertimos todos los
  pos = STRALLPOS(format5,'I') ;  enteros sencillos en largos para evitar 
  IF pos[0] NE -1 THEN BEGIN   ;  problemas
   FOR i=0,N_ELEMENTS(pos)-1 DO STRPUT, format5, 'L', pos[i]
  ENDIF
 ENDIF
 FOR j=0,nvar-1 DO BEGIN
  CASE STRMID(format5,j,1) OF
   'A': idltype[j] = 7
   'D': idltype[j] = 5
   'F': idltype[j] = 4
   'E': idltype[j] = 5
   'I': idltype[j] = 2
   'B': idltype[j] = 1
   'L': idltype[j] = 3
  ENDCASE
 ENDFOR
 IF fullf EQ 1 THEN BEGIN
  ww     = 'w' + STRTRIM(INDGEN(nvar)+1,2)
  wchain = 'w1'
  FOR j=0,nvar-1 DO BEGIN
;   st  = ww[j] + ' = ""'
   st  = ww[j] + ' = MAKE_ARRAY(   1,TYPE=idltype[j])'
   tst = EXECUTE(st)
   IF j NE 0 THEN wchain = wchain + ',' + ww[j]
  ENDFOR
 ENDIF
ENDIF ELSE BEGIN
 IF KEYWORD_SET(ONESTRING) THEN idltype[*] = 7 $ 
                           ELSE idltype[*] = 5 ; Doble precisin por defecto
 fullf      = 0
ENDELSE
numeric = (idltype NE 7)
;
; Creacin de variables
;
FOR j=0,nvar-1 DO BEGIN
 st  = vv[j] + ' = MAKE_ARRAY(nmax,TYPE=idltype[j])'
 tst = EXECUTE(st)
ENDFOR
text = ''
;
; Abrimos el fichero y lo comprobamos
;
OPENR, unit, file, /GET_LUN, ERROR=err
IF err NE 0 THEN BEGIN
 IF quiet LT 2 THEN PRINT, 'READDATA failed, file problem'
 CLOSE, unit
 FREE_LUN, unit
 RETURN
ENDIF
;
; Bucle de lectura de datos
;
n  = 0L
vs = ''
FOR i=0L,nmax-1 DO BEGIN
 READF, unit, vs
;
; Eliminacin de tabuladores
;
 pos1  = STRPOS(vs,STRING(9b)) 
 WHILE pos1 NE -1 DO BEGIN
  vs   = STRMID(vs,0,pos1) + ' ' + STRMID(vs,pos1+1,STRLEN(vs))
  pos0 = pos1
  pos1 = STRPOS(vs,STRING(9b),pos0+1) 
 ENDWHILE
;
; Distribucin por casos
;
 message = ''
 newtext = vs
 IF STRLEN(vs) LT nvar THEN BEGIN
  message = 'kk'
  GOTO, NODATA
 ENDIF
 IF KEYWORD_SET(SKIPCH) AND STRMID(vs,0,MAX([STRLEN(skipch),1])) EQ skipch THEN BEGIN
  message = 'kk'
  GOTO, NODATA
 ENDIF
 IF KEYWORD_SET(ONESTRING) THEN BEGIN
  IF KEYWORD_SET(STRTRIM) THEN v1[n] = STRTRIM(vs,2) ELSE v1[n] = vs
  GOTO, DATA
 ENDIF
 vs0 = vs
 IF fullf EQ 0 THEN BEGIN ; Formateado simple
  vs  = STRTRIM(vs,2)
  IF since_v53 THEN var = STRSPLIT(STRCOMPRESS(vs),sepch,/EXTRACT,/PRESERVE_NULL) $
               ELSE var =  STR_SEP(STRCOMPRESS(vs),sepch)
;  var = STR_SEP(STRCOMPRESS(vs),sepch)
  IF N_ELEMENTS(var) GT nvar AND N_ELEMENTS(var) GT 1 AND KEYWORD_SET(STRUCTURE) AND sepch NE ' ' AND flag EQ 0 AND fullf EQ 0 THEN BEGIN ; Aadido automtico de columnas de caracteres
    flag = 1
    CLOSE, unit
    FREE_LUN, unit
    aux     = STRJOIN('A' + STRARR(N_ELEMENTS(var)-nvar))
    format5 = format5 + aux
    nvar    = N_ELEMENTS(var)
    GOTO, AUTOFORMAT
  ENDIF
  IF N_ELEMENTS(var) LT nvar THEN BEGIN
   message = 'READDATA problem, line too short at line number ' + STRTRIM( i+1,2)
   GOTO, NODATA
  ENDIF
  IF KEYWORD_SET(STRTRIM) THEN var = STRTRIM(var,2)
  FOR j=0,nvar-1 DO BEGIN
   IF numeric[j] THEN BEGIN
;    tst = STRNUMBER(var[j],val)
    tst = VALID_NUM(var[j],val)
    IF tst EQ 0 THEN BEGIN
     message = 'READDATA problem, format conversion at line number ' + STRTRIM( i+1,2)
     GOTO, NODATA
    ENDIF
    st = vv[j] + '[n] = val'
   ENDIF ELSE BEGIN
    st = vv[j] + '[n] = var[j]'
   ENDELSE
   tst = EXECUTE(st)
  ENDFOR
 ENDIF ELSE BEGIN         ; Formateado complejo
  CATCH, error_status, /CANCEL
  MESSAGE, /RESET_ERROR_STATE
  ON_IOERROR, NODATA
  message =  'READDATA problem, format conversion at line number ' +STRTRIM( i+1,2)
  st  = 'READS, vs, ' + wchain + ', FORMAT=formate'
  tst = EXECUTE(st)
  IF !ERR_STRING NE '' THEN GOTO, NODATA
  message = ''
  FOR j=0,nvar-1 DO BEGIN
   st  = vv[j] + '[n] = ' + ww[j]
   tst = EXECUTE(st)
  ENDFOR
 ENDELSE
 DATA:
 n = n + 1
 NODATA:
 IF message NE '' THEN BEGIN
  IF N_ELEMENTS(text) EQ structure AND N_ELEMENTS(var) GE nvar THEN BEGIN
   tags = SUBSTR(STRTRIM(var[0:nvar-1],2) ,[' ','-','+','#','.','(',')','2MASS'],['_','','','num','_','','','TWOMASS'])
   p    = WHERE(tags EQ 'mod',np)
   IF np NE 0 THEN tags[p] = 'model' 
   FOR j=1,nvar-1 DO FOR k=0,j-1 DO IF STRLOWCASE(tags[k]) EQ STRLOWCASE(tags[j]) THEN tags[k] = tags[k] + '_' + STRTRIM(k+1,2)
  ENDIF
  text = [text, newtext]
  IF quiet LT 1 AND message NE 'kk' THEN PRINT, message
 ENDIF
ENDFOR
;
; Recorte de variables
;
np = n
IF n EQ 0 THEN BEGIN
 IF quiet LT 2 THEN PRINT, 'READDATA failed, no valid lines'
 err = 1
 RETURN
ENDIF
FOR i=0,nvar-1 DO BEGIN
 st  = vv[i] + ' = ' + vv[i] + '[0:n-1]'
 tst = EXECUTE(st)
ENDFOR
IF N_ELEMENTS(text) NE 1 THEN text = text[1:N_ELEMENTS(text)-1]
IF KEYWORD_SET(STRUCTURE) THEN BEGIN
 st  = 'aux = {' + STRJOIN(tags + ':' + vv, ',') + '}'
 tst = EXECUTE(st)
 v1  = aux
ENDIF
CLOSE, unit
FREE_LUN, unit
END
;</PRE>
