;+
; SINGLEPLOT generates a Postscript 1-D plot with as many as 16 data sets.
; See the <a HREF="../singleplot.pro">full procedure</a><BR>
; @examples <PRE>
;  A simple call to SINGLEPLOT is:
;   x1 = FINDGEN(101)
;   y1 = x1 + x1^2 - 0.01*x1^3 + 10*RANDOMN(seed,101)
;   SINGLEPLOT, "test01.ps", {x:x1, y:y1}
;  See the <a HREF="jmaplot_overview.html#tutorial" TARGET="parent">jmaplot tutorial</a> for additional examples. </PRE>
; @version v. 3.1, 11 May 2013.                                             <BR>
; @history <PRE>
;
;  v0.0: First version.
;  v0.1: XLESSTICKS and YLESSTICKS introduced.
;        EXTRAX and EXTRAY can be given specific values.
;        XC and YC added as input variables.
;        More flexibility added to XSIZE, YSIZE, and PSASPECT.
;        New plot positioning algorithm.
;        Multiple data sets added.
;  v0.2: Error plots introduced.
;        SYMSIZE added.
;        legends added.
;        MINMAX2 as a separate procedure.
;        The preparation and opening of the PostScript file are moved to
;         PREPPS and OPENPS.
;  v0.3: PLOTTYPE is added to increase the funtionality and the data sets are
;         changed to be introduced as v1, v2, v3... rather than as
;         x1, y1, y2...
;        A bug in the calculation of PSASPECT is corrected.
;        DAXIS is introduced.
;        PSXSIZE is introduced to allow its call from MULTIGRAPH.
;        LTYPE is added.
;        Arbitrary symbols for error plots are introduced.
;        The procedure is reorganized for compatibility with SINGLECONTOUR
;         and MULTIGRAPH.
;  v0.4: EXTERNAL is introduced.
;        LANDSCAPE is introduced.
;        Negative values for PSYM allowed.
;        Adjusted procedure for LEGEND v0.1.
;        XLOG and YLOG = 2 allowed.
;  v0.5: XMORETICKS and YMORETICKS introduced.
;        NOFILL introduced.
;        EXTXTICKS and EXTYICKS introduced.
;        EXTRASEPXAXIS introduced.
;        A bug with THICK is fixed.
;  v1.0: General clean up.
;  v1.1: LCLEAR added (LEGEND v1.1).
;        BACKGROUND added.
;        PLOTTYPE=7 added.
;        System variables saved.
;        Additinal options for DAXIS.
;        PREEXTERNAL added.
;  v1.2: PSFONTS added.
;        CFILE='colorjma.tbl' and CTABLE=40 are made default values.
;  v1.3: Maximum number of plots increased from 10 to 16.
;        Some legend keyword are passed to _REF_EXTRA
;  v1.3: PLOTTYPE=9 added.
;  v2.0: English version.
;  v2.1: IDLdoc documentation.
;        CTABLE problems from MULTIGRAPH eliminated.
;        Addition of new PSYM values and displacement of lines and limits to
;         upper values.
;        Fixed problems with filled histograms.
;        Fix to the small font problem when using MULTIGRAPH (NPX).
;        Fixed problems with XLOG or YLOG = 2.
;  v2.2: OPENHTML introduced.
;        Upgrade to IDLdoc documentation.
;  v2.3: To ensure compatibility, _REF_EXTRA changed to _EXTRA.
;        Eliminated the NOOPAXIS obsolete keyword.
;        Windows compatibility enhanced.
;        CLOSEPS bug fixed.
;  v2.4: COMMON singleplotvars added.
;  v2.5: Version system update.
;        Maximum number of plots increased to 60.
;        QUIET flag added.             
;  v3.0: Possibility of using structures as data input (arrays
;         become obsolete). This allows for greater flexibility
;         in differentiating data.
;        Axis, plot, and color file information restructured with 
;         XL, YL, XU, YU, PLOT, and CFILE structures. This allows
;         for more diverse axes, such as different lower and 
;         upper ones.
;        New horizontal+vertical error bars with angle.
;        New arrow plots.
;        New grid, thick, and linestyle options for axes.
;        Improvement of the interaction with other IDL (external)
;         routines such as MAP_SET.
;        Introduction of IN and OUT structures.
;        Use of most old keywords marked as obsolete.
;        New variable jmaplot_path
;        colorjma.tbl in jmaplot main directory and other 
;         improvements to color tables.
;        MAXPOINTS eliminated.                            
;        PLOTTYPE eliminated.                            
;        PSFONTS eliminated (now the default value).                            
;        EXTRASEPXAXIS deactivated.                       
;        COMMON singleplotvars eliminated.
;        Histogram with border (psym=12 added).     
;        CMYK PS output added.
;        HTML pages moved to local directory.
;        MYDEVICE eliminated.
; v3.1:  NEGATIVE added.
;        NOBORDER added to LEGEND.  </PRE>
; @author Jess Maz Apellniz, IAA               <BR>
; @param   psname {in}{required}{type=string} Postscript file name.
;           If '' is passed, the Postscript file is
;           neither opened nor closed (this option is intended to be used
;           from MULTIGRAPH).                                               <BR>
; @param   v1 {in}{optional}{type=structure} First data structure. It is 
;           required if the BACKGROUND keyword is not used. Possible tags: <PRE>
;           X:         x values.
;           Y:         y values (required).
;           SX:        x uncertainty values.
;           SY:        y uncertainty values.
;           SXM:       x inferior uncertainty values.
;           SXP:       x superior uncertainty values.
;           SYM:       y inferior uncertainty values.
;           SYP:       y superior uncertainty values.
;           ANG:       CCW angle in degrees between y uncertainties 
;                       and vertical axis.
;           TXT:       Text to be written with XYOUTS.
;           TXM:       XYOUTS modifiers. 
;           DX:        Delta x for arrows.
;           DY:        Delta y for arrows.
;           HEADL:     Arrowhead length.    
;           ARROWT:    Arrow thickness.
;           HEADT:     Arrowhead thickness.
;           COLOR:     Color index.
;           LINESTYLE: IDL line style.
;           PSYM:      IDL symbol code.The following non-standard values are 
;                       accepted: 
;       11: Filled histogram, no border
;       12: Filled histogram, black (text) border
;      100: circle         101: upright triangle  102: inverted triangle
;      103: square         104: diamond           105: simple cross
;      106; pentagon       107: hexagon           108: simple star
;      109: star of David  110: complex cross     111: Sun
;      112: crescent Moon  113: Earth symbol      114: heart
;      120: vertical line  121: horizontal line   130: lower limit
;      131: upper limit    132: right limit       133: left limit
;      134-37: limits in the four quadrants
;           THICK:     Line thicknesses for each data set.
;           SYMSIZE:   Symbol sizes for each data set.
;           NOFILL:    Flag to use with GENSYM. </PRE>
; @param   v2 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v3 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v4 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v5 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v6 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v7 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v8 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v9 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v10 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v11 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v12 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v13 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v14 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v15 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v16 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v17 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v18 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v19 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v20 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v21 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v22 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v23 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v24 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v25 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v26 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v27 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v28 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v29 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v30 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v31 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v32 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v33 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v34 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v35 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v36 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v37 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v38 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v39 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v40 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v41 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v42 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v43 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v44 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v45 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v46 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v47 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v48 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v49 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v50 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v51 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v52 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v53 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v54 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v55 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v56 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v57 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v58 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v59 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @param   v60 {in}{optional}{type=structure} Additional data structure. See v1 for tags. <BR>
; @keyword BACKGROUND {in}{optional}{type=byte array} Background image of
;           dimensions (nx,ny) or (nx,ny,3).             <BR>
; @keyword XAXISL {in}{optional}{type=structure} Lower x-axis structure with the 
;           following possible tags: <PRE>
;           RANGE:     2-element float array with minimum and maximum.
;           EXTRA:     If -1.0, it increases the x range by 10%. If not 1.0, it
;                       increases the natural x range by that fraction. RANGE
;                       supersedes it.
;           LOG:       Flag for logarithmic axis. A value of 2 does not force 
;                       scientific notation as 1 does.
;           STYLE:     Axis type, -2 (nothing), -1 (lines), 0 (lines and
;                       ticks), and 1 (lines, ticks, and text). 
;           TITLE:     Axis title.                       
;           THICK:     Line thickness.                   
;           LINESTYLE: Line style.                       
;           TICKNAME:  Text for the tick marks.            
;           CHARSIZE:  Text size (also affected by PLOT.CHARSIZE).
;           TICKV:     Values for the tick marks.            
;           TICKLEN:   Scale factor for the tick lengths.
;           LESSTICKS: Flag for using less tick marks (ignored if TICKV given).
;           MORETICKS: Flag for using more tick marks (ignored if TICKV given). </PRE>
;           MINORN:    Number of minor tick marks per major ones, counting major 
;                       ones as minor (ignored if LOG is activated).
; @keyword YAXISL {in}{optional}{type=structure} Lower y-axis structure. See XAXISL
;           for the possible tags: <BR>
; @keyword XAXISU {in}{optional}{type=structure} Upper x-axis structure. See XAXISL
;           for the possible tags: <BR>
; @keyword YAXISU {in}{optional}{type=structure} Upper y-axis structure. See XAXISL
;           for the possible tags: <BR>
; @keyword PLOT {in}{optional}{type=structure} Plot structure with the 
;           following possible tags: <PRE>
;           XSIZE:     X size (in normal coordinates) of the plot.
;           YSIZE:     Y size (in normal coordinates) of the plot.
;           XC:        X position (in normal coordinates) of the center of the plot.
;           YC:        Y position (in normal coordinates) of the center of the plot.
;           CHARSIZE:  Global text size.
;           CHARTHICK: Global text thickness.
;           PSASPECT:  Aspect ratio of the Postscript file.
;           PSXSIZE:   X size (in cm) of the Postscript file.
;           LANDSCAPE: Flag for landscape mode.
;           FONT:      -1 (Vector-drawn), 0 (PS), 1 (TrueType).
;           CMYK:      Flag to generate CMYK PS output.
;           NEGATIVE:  Flag to generate a negative output (black and white exchanged).
;           TITLE:     Main title.  </PRE>
; @keyword CFILE {in}{optional}{type=string} Color table structure with
;           the following possible tags: <PRE>
;           NAME:      File name.                                  
;           DIR:       Directory name.                             
;           TABLE:     Color table.                                
;           DEF:       Default color. </PRE>
; @keyword LEGEND {in}{optional}{type=structure} Legend structure with the 
;           following possible tags: <PRE>
;           TEXT:      Legend text.
;           CLEAR:     Flag to clear the legend background.
;           XLEFT:     Left border of the legend box in plot-normalized
;                       coordinates (default: 0.05).
;           YDOWN:     Lower border of the legend box in plot-normalized
;                       coordinates (default: 0.05).
;           XSIZE:     Horizontal size of the legend box in plot-normalized
;                       coordinates (default: 0.25).
;           YSIZE:     Horizontal size of the legend box in plot-normalized
;                       coordinates (default: 0.15).
;           NOBORDER:  Flag not to draw a border around the legend.</PRE>
; @keyword SPANISH {in}{optional}{type=boolean}{default=FALSE} Flag to use
;           decimal commas instead of decimal points.            <BR>
; @keyword PREEXTERNAL {in}{optional}{type=string array} String(s) with commands
;           to be executed before the plots are drawn.        <BR>
; @keyword EXTERNAL {in}{optional}{type=string array} String(s) with commands
;           to be executed after the plots are drawn.        <BR>
; @keyword QUIET {in}{optional}{type=boolean}{default=FALSE} Flag to supress informational messages.<BR>
; @keyword IN {in}{optional}{type=structure} Input structure to pass 
;           the structures produced by out. <BR>
; @keyword OUT {out}{optional}{type=structure} Output structure.
; @keyword VAR {in}{optional}{type=structure} Input structure to pass 
;           additional variables. <BR>
; @keyword ENTRANCE {in}{optional}{type=integer} Entrance point for
;           sequential calls from MULTIGRAPH.
; @keyword EXIT {in}{optional}{type=integer} Exit point for sequential
;           calls from MULTIGRAPH.
; @keyword COLOR {in}{optional}{type=byte array}{default=[0...0]} OBSOLETE. Color for
;           each data set. See v1. <BR>
; @keyword LINESTYLE {in}{optional}{type=byte array}{default=[0...0]} OBSOLETE. Line
;           style for each data set. See v1. <BR>
;           a number of elements equal to the number of data sets.<BR>
; @keyword PSYM {in}{optional}{type=byte array}{default=[0...0]} OBSOLETE. Symbol code for
;           each data set. See v1. <BR>
; @keyword THICK {in}{optional}{type=float array}{default=[1...1]} OBSOLETE. Line
;           thicknesses for each data set. See v1. <BR>
; @keyword SYMSIZE {in}{optional}{type=float array}{default=[1...1]} OBSOLETE. Symbol
;           sizes for each data set. See v1. <BR>
; @keyword NOFILL {in}{optional}{type=boolean array}{default=[FALSE...FALSE]}
;           OBSOLETE. Flag to use with GENSYM. See v1. <BR>
; @keyword XTITLE {in}{optional}{type=string} OBSOLETE. X title. See AXISL. <BR>
; @keyword YTITLE {in}{optional}{type=string} OBSOLETE. Y title. See AXISL. <BR>
; @keyword DAXIS {in}{optional}{type=4-element integer array}{default=[1,1,1,1]}
;           OBSOLETE. [yleft, xdown, yright, xup] array with options for the four axes.
;           Allowed values are: -2 (nothing), -1 (lines), 0 (lines and
;           ticks), and 1 (lines, ticks, and text). See AXISL. <BR>
; @keyword TICKF {in}{optional}{type=float} OBSOLETE. Scale factor for the tick lengths. See AXISL.  <BR>
; @keyword EXTRAX {in}{optional}{type=boolean or float}{default=FALSE} OBSOLETE. As
;           boolean (/EXTRAX), it increases the x range by 10%. As a float, it
;           increases the x range by that fraction. See AXISL.                          <BR>
; @keyword EXTRAY {in}{optional}{type=boolean or float}{default=FALSE} OBSOLETE. As
;           boolean (/EXTRAY), it increases the y range by 10%. As a float, it
;           increases the y range by that fraction. See AXISL.                          <BR>
; @keyword XLOG {in}{optional}{type=boolean or integer}{default=FALSE} OBSOLETE. Flag for
;           logarithmic x axis. A value of 2 does not force scientific notation
;           as 1 does. See AXISL. <BR>
; @keyword YLOG {in}{optional}{type=boolean or integer}{default=FALSE} OBSOLETE. Flag for
;           logarithmic y axis. A value of 2 does not force scientific notation
;           as 1 does. See XAXISL. <BR>
; @keyword XLESSTICKS {in}{optional}{type=boolean}{default=FALSE} OBSOLETE. Flag to use
;           less ticks for the x axis. See XAXISL.                           <BR>
; @keyword YLESSTICKS {in}{optional}{type=boolean}{default=FALSE} OBSOLETE. Flag to use
;           less ticks for the y axis. See XAXISL.                           <BR>
; @keyword XMORETICKS {in}{optional}{type=boolean}{default=FALSE} OBSOLETE. Flag to use
;           less ticks for the x axis (do not use for log axes). See XAXISL. <BR>
; @keyword YMORETICKS {in}{optional}{type=boolean}{default=FALSE} OBSOLETE. Flag to use
;           less ticks for the y axis (do not use for log axes). See XAXISL. <BR>
; @keyword MINMAX {in}{optional}{type=4-elements float array}
;          OBSOLETE. [xmin,ymin,xmax,ymax] values for the x and y ranges. See XAXISL.  <BR>
; @keyword EXTXTICKS {in}{optional}{type=string array} OBSOLETE. Text for the x axis
;           ticks. See XAXISL.                 <BR>
; @keyword EXTYTICKS {in}{optional}{type=string array} OBSOLETE. Text for the y axis
;           ticks. See XAXISL.                 <BR>
; @keyword XSIZE {in}{optional}{type=float}{default=0.75-0.82} OBSOLETE. X size (in
;           normal coordinates) of the plot. See PLOT.  <BR>
; @keyword YSIZE {in}{optional}{type=float}{default=0.75-0.82} OBSOLETE. Y size (in
;           normal coordinates) of the plot. See PLOT.  <BR>
; @keyword XC {in}{optional}{type=float}{default=approx. 0.5} OBSOLETE. X position (in
;           normal coordinates) of the center of the plot. See PLOT.  <BR>
; @keyword YC {in}{optional}{type=float}{default=approx. 0.5} OBSOLETE. Y position (in
;           normal coordinates) of the center of the plot. See PLOT.  <BR>
; @keyword AXISF {in}{optional}{type=float} OBSOLETE. Scale factor for the axis fonts. See PLOT.  <BR>
; @keyword PSASPECT {in}{optional}{type=float}{default=1.0} OBSOLETE. Aspect ratio of the
;           Postscript file. See PLOT.  <BR>
; @keyword PSXSIZE {in}{optional}{type=float}{default=19.0} OBSOLETE. X size (in cm) of
;           the Postscript file. See PLOT.  <BR>
; @keyword LANDSCAPE {in}{optional}{type=boolean}{default=FALSE} OBSOLETE. Flag for
;           landscape mode. See PLOT.                                          <BR>
; @keyword TITLE {in}{optional}{type=string} OBSOLETE. Main title. See PLOT. <BR>
; @keyword CTABLE {in}{optional}{type=byte}{default=40 or 0} OBSOLETE. Color table. If
;           CFILE is 'colorjma.tbl', the default value is 40. For other files
;           the default value is 0. See CFILE.                        <BR>
; @keyword CTEXT {in}{optional}{type=byte}{default=0 or 254} OBSOLETE. Text and axes
;           color. Note that the plot color is set with COLOR. The default
;           value is 254 only if CFILE='colorjma.tbl' and CTABLE=39. See CFILE. <BR>
; @keyword LTYPE {in}{optional}{type=integer}{default=1} OBSOLETE. Legend type: 1
;           (standard) or 2 (only text). See LEGEND.       <BR>
; @keyword LCLEAR {in}{optional}{type=boolean}{default=FALSE} OBSOLETE. Flag to clear the legend background. See LEGEND. <BR>
; @keyword LXLEFT {in}{optional}{type=float} OBSOLETE. Left border of the legend box in plot-normalized
;            coordinates (default: 0.05). See LEGEND. <BR>
; @keyword LYDOWN {in}{optional}{type=float} OBSOLETE. Lower border of the legend box in plot-normalized
;            coordinates (default: 0.05). See LEGEND. <BR>
; @keyword LXSIZE {in}{optional}{type=float} OBSOLETE. Horizontal size of the legend box in plot-normalized
;            coordinates (default: 0.25). See LEGEND. <BR>
; @keyword LYSIZE {in}{optional}{type=float} OBSOLETE. Vertical size of the legend box in plot-normalized
;            coordinates (default: 0.15). See LEGEND. <BR>
;-
;<PRE>
PRO SINGLEPLOT, psname, $
	            v1,  v2,  v3,  v4,  v5,  v6,  v7,  v8,  v9, v10, $
                   v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, $
                   v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, $
                   v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, $
                   v41, v42, v43, v44, v45, v46, v47, v48, v49, v50, $
                   v51, v52, v53, v54, v55, v56, v57, v58, v59, v60, $
		   BACKGROUND=background, $
		   XAXISL=xaxisl, YAXISL=yaxisl, $
		   XAXISU=xaxisu, YAXISU=yaxisu, $
		   PLOT=plot, CFILE=cfile, LEGEND=legend, $
		   SPANISH=spanish, $
           	   PREEXTERNAL=preexternal, EXTERNAL=external, $
                   QUIET=quiet, $
		   IN=in, OUT=out, VAR=var, $
		   ENTRANCE=entrance, EXIT=exit, $
		   COLOR=color, LINESTYLE=linestyle, PSYM=psym, $
		   NOFILL=nofill, THICK=thick, SYMSIZE=symsize, $
		   XTITLE=xtitle, YTITLE=ytitle, $
		   DAXIS=daxis, $
		   TICKF=tickf, $
                   EXTRAX=extrax, EXTRAY=extray, $
                   XLOG=xlog, YLOG=ylog, $
                   XLESSTICKS=xlessticks, YLESSTICKS=ylessticks, $
                   XMORETICKS=xmoreticks, YMORETICKS=ymoreticks, $
                   MINMAX=minmax, $
                   EXTXTICKS=extxticks, EXTYTICKS=extyticks, $
                   XSIZE=xsize, YSIZE=ysize, XC=xc, YC=yc, $
		   AXISF=axisf, $
		   PSASPECT=psaspect, $
                   PSXSIZE=psxsize, LANDSCAPE=landscape, $
                   TITLE=title, $
                   CTABLE=ctable, CTEXT=ctext, $
	           LTYPE=ltype, LCLEAR=lclear, $
                   LXLEFT=lxleft, LYDOWN=lydown, LXSIZE=lxsize, LYSIZE=lysize
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;          (a) Inicializacin general                                          ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Inicializamos variables
;
COMMON vv,          vv1,  vv2,  vv3,  vv4,  vv5,  vv6,  vv7,  vv8,  vv9, vv10, $
                   vv11, vv12, vv13, vv14, vv15, vv16, vv17, vv18, vv19, vv20, $
                   vv21, vv22, vv23, vv24, vv25, vv26, vv27, vv28, vv29, vv30, $
                   vv31, vv32, vv33, vv34, vv35, vv36, vv37, vv38, vv39, vv40, $
                   vv41, vv42, vv43, vv44, vv45, vv46, vv47, vv48, vv49, vv50, $
                   vv51, vv52, vv53, vv54, vv55, vv56, vv57, vv58, vv59, vv60
ON_ERROR, 1
jmaplot_path = JMAPLOTPATH(JMAPLOT_VERSION=jmaplot_version)
nplots  = N_PARAMS()-1
IF nplots EQ -1 THEN BEGIN
 OPENHTML, jmaplot_path + 'html/singleplot.html'
 RETURN
ENDIF
IF N_ELEMENTS(IN) NE 0 THEN BEGIN
 IF TAG_EXIST(in,'nplots') THEN BEGIN
  nplots = in.nplots
 ENDIF ELSE BEGIN
  aux    = STRLOWCASE(TAG_NAMES(in))
  pos    = WHERE(STRMID(aux,0,1) EQ 'v' AND aux NE 'var', nplots)
  IF nplots NE 0 THEN BEGIN
   aux2  = MAX(FIX(STRMID(aux[pos],1,999)))
   IF aux2 NE nplots THEN MESSAGE, 'Incorrect tags in singleplot call'
  ENDIF
 ENDELSE
 FOR i=0,nplots-1 DO tmp = EXECUTE('v'  + STRTRIM(i+1,2) + '=in.v' + STRTRIM(i+1,2))
 FOR i=0,nplots-1 DO tmp = EXECUTE('vv' + STRTRIM(i+1,2) + '=in.v' + STRTRIM(i+1,2))
 IF TAG_EXIST(in,'background')  THEN background  = in.background
 IF TAG_EXIST(in,'spanish'   )  THEN spanish     = in.spanish 
 IF TAG_EXIST(in,'preexternal') THEN preexternal = in.preexternal
 IF TAG_EXIST(in,'external')    THEN external    = in.external
 IF TAG_EXIST(in,'var')         THEN var         = in.var
ENDIF
nplotsr = nplots
xsv     = !X
ysv     = !Y
psv     = !P
IF KEYWORD_SET(BACKGROUND)  EQ 0 THEN background  = 0
IF KEYWORD_SET(SPANISH)     EQ 0 THEN spanish     = 0
IF KEYWORD_SET(PREEXTERNAL) EQ 0 THEN preexternal = ''
IF KEYWORD_SET(EXTERNAL)    EQ 0 THEN external    = ''
IF KEYWORD_SET(VAR)         EQ 0 THEN var         = {dummy:0}
IF KEYWORD_SET(ENTRANCE)    EQ 0 THEN entrance    = 0
IF KEYWORD_SET(EXIT)        EQ 0 THEN exit       = 0
IF KEYWORD_SET(CFILE)       EQ 0 THEN cfileo      = ''     ELSE $
	                              cfileo      = cfile
IF KEYWORD_SET(LEGEND)      EQ 0 THEN legendo     = ''     ELSE $
	                              legendo     = legend
;
; Parte en comn con SINGLECONTOUR
;
COMMONSINGLE, xxl, yyl, xxu, yyu, pplot, nplots, v1, $
              BACKGROUND=background, $
	      XAXISL=xaxisl, YAXISL=yaxisl, $
	      XAXISU=xaxisu, YAXISU=yaxisu, $
	      PLOT=plot, CFILE=cfile, LEGEND=legend, QUIET=quiet, IN=in,   $
	      DAXIS=daxis, MINMAX=minmax, $
	      PSASPECT=psaspect, PSXSIZE=psxsize, $
	      LANDSCAPE=landscape, PSFONTS=psfonts
IF entrance EQ 1 THEN GOTO, ENTRANCE1
IF entrance EQ 2 THEN GOTO, ENTRANCE2
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;          (b) Inicializacin de datos y propiedades (V...)                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
IF KEYWORD_SET(COLOR) +KEYWORD_SET(LINESTYLE)+KEYWORD_SET(THICK)+KEYWORD_SET(PSYM)+ $
   KEYWORD_SET(NOFILL)+KEYWORD_SET(SYMSIZE) NE 0 AND KEYWORD_SET(QUIET) EQ 0 THEN $
   PRINT, 'Warning: using obsolete data keywords!'
;
; Bucle principal
;
FOR i=0,nplots-1 DO BEGIN
 is       = STRTRIM(i+1,2)
 vis      = 'v' + is
 tmp      = EXECUTE('aux=SIZE(' + vis + ',/TNAME)')
;
; Los datos son variables
;
 IF aux NE 'STRUCT' THEN BEGIN
  IF KEYWORD_SET(QUIET) EQ 0 THEN PRINT, 'Warning: using arrays as input data is obsolete!'
; Procesado de los datos
  tmp    = EXECUTE('v = ' + vis)
  aux    = SIZE(v)
  IF aux[0] EQ 1 THEN BEGIN
   v     = [[FINDGEN(aux[1])],[v]]
   aux   = SIZE(v)
  ENDIF
  IF aux[0] NE 2 THEN MESSAGE, 'Invalid ' + vname + ' data'
  ncolsi = aux[2]
  nptsi  = aux[1]
  vtype  = SIZE(v,/TYPE)
  IF vtype GE 1 AND vtype LE 3 THEN v = FLOAT(v)
  plottype = 0 ; plottype desactivado
;  IF KEYWORD_SET(PLOTTYPE) THEN BEGIN
;   IF N_ELEMENTS(plottype) GE i+1 THEN plottypei = plottype[i] $
;	                          ELSE plottypei = plottype[0]
;   IF (plottypei EQ 1 AND ncolsi NE 2) OR $
;      (plottypei EQ 2 AND ncolsi NE 3) OR $
;      (plottypei EQ 3 AND ncolsi NE 3) OR $
;      (plottypei EQ 4 AND ncolsi NE 4) OR $
;      (plottypei EQ 5 AND ncolsi NE 4) OR $
;      (plottypei EQ 6 AND ncolsi NE 4) OR $
;      (plottypei EQ 7 AND ncolsi NE 5) OR $
;      (plottypei EQ 8 AND ncolsi NE 6) OR $
;      (plottypei EQ 9 AND ncolsi NE 4) OR $
;      (plottypei EQ 9 AND vtype  NE 7) THEN MESSAGE, 'Invalid data'
;  ENDIF ELSE BEGIN
   IF vtype  EQ 7 AND ncolsi NE 3 AND ncolsi NE 4 THEN MESSAGE, 'Invalid data'
   IF                 ncolsi LT 2 OR  ncolsi GT 6 THEN MESSAGE, 'Invalid data'
   IF vtype  EQ 7 THEN plottypei = 9 ELSE $
   IF ncolsi EQ 2 THEN plottypei = 1 ELSE $
   IF ncolsi EQ 3 THEN plottypei = 3 ELSE $
   IF ncolsi EQ 4 THEN plottypei = 4 ELSE $
   IF ncolsi EQ 5 THEN plottypei = 7 ELSE $
                       plottypei = 8
;  ENDELSE
  aux = FLTARR(nptsi)
  CASE plottypei OF
   1: v = {x:v[*,0], y:v[*,1], sxm:aux   , sxp:aux   , sym:aux   , syp:aux   , ang:aux   , n:nptsi, ert:0} 
   2: v = {x:v[*,0], y:v[*,1], sxm:v[*,2], sxp:v[*,2], sym:aux   , syp:aux   , ang:aux   , n:nptsi, ert:1}
   3: v = {x:v[*,0], y:v[*,1], sxm:aux   , sxp:aux   , sym:v[*,2], syp:v[*,2], ang:aux   , n:nptsi, ert:2}
   4: v = {x:v[*,0], y:v[*,1], sxm:v[*,2], sxp:v[*,2], sym:v[*,3], syp:v[*,3], ang:aux   , n:nptsi, ert:3}
   5: v = {x:v[*,0], y:v[*,1], sxm:v[*,3], sxp:v[*,2], sym:aux   , syp:aux   , ang:aux   , n:nptsi, ert:1}
   6: v = {x:v[*,0], y:v[*,1], sxm:aux   , sxp:aux   , sym:v[*,3], syp:v[*,2], ang:aux   , n:nptsi, ert:2}
   7: v = {x:v[*,0], y:v[*,1], sxm:aux   , sxp:aux   , sym:v[*,3], syp:v[*,2], ang:v[*,4], n:nptsi, ert:4}
   8: v = {x:v[*,0], y:v[*,1], sxm:v[*,3], sxp:v[*,2], sym:v[*,5], syp:v[*,4], ang:aux   , n:nptsi, ert:3}
   9: BEGIN
       IF ncolsi EQ 4 THEN aux = v[*,3] ELSE aux = 'ALIGNMENT=0.5, /DATA' + STRARR(nptsi)
       v = {x:DOUBLE(v[*,0]), y:DOUBLE(v[*,1]), sxm:aux, sxp:aux, sym:aux, syp:aux, ang:aux, $
	    txt:v[*,2], txm:aux, n:nptsi, ert:6}
      END
  ENDCASE
  v = CREATE_STRUCT(v, {dx:FLTARR(v.n), dy:FLTARR(v.n)})
; Procesado de los propiedades
  colori     = 0
  IF KEYWORD_SET(COLOR    )      THEN BEGIN
   IF N_ELEMENTS(color    ) GT i THEN colori     =    color[i] $
	                         ELSE colori     =    color[0]
  ENDIF
  v = CREATE_STRUCT(v, {    color:INTARR(v.n) +     colori})
  linestylei = 0
  IF KEYWORD_SET(LINESTYLE)      THEN BEGIN
   IF N_ELEMENTS(linestyle) GT i THEN linestylei =linestyle[i] $
	                         ELSE linestylei =linestyle[0]
  ENDIF
  v = CREATE_STRUCT(v, {linestyle:              linestylei})
  thicki     = 1.0
  IF KEYWORD_SET(THICK    )      THEN BEGIN
   IF N_ELEMENTS(thick    ) GT i THEN thicki     =    thick[i] $
	                         ELSE thicki     =    thick[0]
  ENDIF
  v = CREATE_STRUCT(v, {    thick:FLTARR(v.n) +     thicki})
  psymi      = 0
  IF KEYWORD_SET(PSYM     )      THEN BEGIN
   IF N_ELEMENTS(psym     ) GT i THEN psymi      =     psym[i] $
	                         ELSE psymi      =     psym[0]
  ENDIF
  v = CREATE_STRUCT(v, {     psym:INTARR(v.n) +      psymi})
  nofilli    = 0
  IF KEYWORD_SET(NOFILL   )      THEN BEGIN
   IF N_ELEMENTS(nofill   ) GT i THEN nofilli    =   nofill[i] $
	                         ELSE nofilli    =   nofill[0]
  ENDIF
  v = CREATE_STRUCT(v, {   nofill:INTARR(v.n) +    nofilli})
  symsizei   = 1.0
  IF KEYWORD_SET(SYMSIZE  )      THEN BEGIN
   IF N_ELEMENTS(symsize  ) GT i THEN symsizei   =  symsize[i] $
	                         ELSE symsizei   =  symsize[0]
  ENDIF
  v = CREATE_STRUCT(v, {  symsize:FLTARR(v.n) +   symsizei})
;
; Los datos son estructuras
;
 ENDIF ELSE BEGIN
  tmp      = EXECUTE('v = ' + vis)
  IF TAG_EXIST(v,'ert' ) EQ  0 THEN v = CREATE_STRUCT(v, {ert:-1}) ; El tipo de error no se ha determinado todava
; x, y 
  IF TAG_EXIST(v,'y'   ) EQ  0 THEN MESSAGE, 'Structure without y tag'
  v_n      = N_ELEMENTS(v.y)
  IF TAG_EXIST(v,'n'   ) EQ  0 THEN v = CREATE_STRUCT(v, {n:v_n}) ELSE v.n = v_n
  IF TAG_EXIST(v,'x'   ) EQ  0 THEN v = CREATE_STRUCT(v, {x:FINDGEN(v.n)})
  v_n2     = N_ELEMENTS(v.x)
  IF v.n  EQ 1 AND v_n2 NE 1 THEN BEGIN
   v.n     = v_n2
   COPY_STRUC, v, v, EXCEPT_TAGS='y'  , STRUC_ADD={  y:v.y   + FLTARR(v.n)}
  ENDIF
  IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='x'  , STRUC_ADD={  x:v.x   + FLTARR(v.n)} $
   ELSE IF   v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible x and y'
  COPY_STRUC, v, v, EXCEPT_TAGS=['x','y'], STRUC_ADD={x:REFORM(v.x),y:REFORM(v.y)}
  v_ert    = 0
; Incertidumbres en x
  IF TAG_EXIST(v,'sx'  ) NE  0 THEN BEGIN              ; Incertidumbres en x+-
   v_ert   = v_ert + 1
   IF TAG_EXIST(v,'sxm') NE  0 OR $
      TAG_EXIST(v,'sxp') NE  0 THEN MESSAGE, 'Structure with incompatible sx and sxm/sxp'
   v_n2    = N_ELEMENTS(v.sx)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='sx' , STRUC_ADD={ sx:v.sx  + FLTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible sx and y'
   v       = CREATE_STRUCT(v, {sxm:v.sx, sxp:v.sx})
  ENDIF ELSE IF TAG_EXIST(v,'sxm') NE  0 OR  $         ; Incertidumbres en x+, x-
                TAG_EXIST(v,'sxp') NE  0 AND $
		(v.ert EQ -1 OR v.ert EQ 1 OR v.ert EQ 3 OR v.ert EQ 5) THEN BEGIN
   IF TAG_EXIST(v,'sxm') EQ  0 OR $
      TAG_EXIST(v,'sxp') EQ  0 THEN MESSAGE, 'Structure with missing sxm or sxp'
   v_ert   = v_ert + 1
   v_n2    = N_ELEMENTS(v.sxm)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='sxm', STRUC_ADD={sxm:v.sxm + FLTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible sxm and y'
   v_n2    = N_ELEMENTS(v.sxp)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='sxp', STRUC_ADD={sxp:v.sxp + FLTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible sxp and y'
  ENDIF
; Incertidumbres en y
  IF TAG_EXIST(v,'sy'  ) NE  0 THEN BEGIN              ; Incertidumbres en y+-
   v_ert   = v_ert + 2
   IF TAG_EXIST(v,'sym') NE  0 OR $
      TAG_EXIST(v,'syp') NE  0 THEN MESSAGE, 'Structure with incompatible sy and sym/sxp'
   v_n2    = N_ELEMENTS(v.sy)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='sy' , STRUC_ADD={ sy:v.sy  + FLTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible sy and y'
   v       = CREATE_STRUCT(v, {sym:v.sy, syp:v.sy})
  ENDIF ELSE IF TAG_EXIST(v,'sym') NE  0 OR  $ ; Incertidumbres en y+, y-
                TAG_EXIST(v,'syp') NE  0 AND $
		(v.ert EQ -1 OR v.ert EQ 2 OR v.ert EQ 3 OR v.ert EQ 4 OR v.ert EQ 5) THEN BEGIN
   IF TAG_EXIST(v,'sym') EQ  0 OR $
      TAG_EXIST(v,'syp') EQ  0 THEN MESSAGE, 'Structure with missing sym or syp'
   v_ert   = v_ert + 2
   v_n2    = N_ELEMENTS(v.sym)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='sym', STRUC_ADD={sym:v.sym + FLTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible sym and y'
   v_n2    = N_ELEMENTS(v.syp)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='syp', STRUC_ADD={syp:v.syp + FLTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible syp and y'
  ENDIF
; Incertidumbres inclinadas
  IF TAG_EXIST(v,'ang')  NE  0 AND $                   ; Incertidumbres inclinadas
     (v.ert EQ -1 OR v.ert EQ 4 OR v.ert EQ 5) THEN BEGIN
   IF TAG_EXIST(v,'sxm') EQ  0 AND $
     (v.ert EQ -1 OR v.ert EQ 4) THEN v_ert = 4 ELSE v_ert = 5
   IF TAG_EXIST(v,'sym') EQ  0 THEN MESSAGE, 'Why does your structure have a rotation angle but no uncertainties in y?'
   v_n2    = N_ELEMENTS(v.ang)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='ang', STRUC_ADD={ang:v.ang + FLTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible ang and y'
  ENDIF
; Flechas
  IF TAG_EXIST(v,'dx') + TAG_EXIST(v,'dy') EQ 1 THEN $
   MESSAGE, 'Why does your structure have only one of xend and yend?'
  IF TAG_EXIST(v,'dx')      NE 0 AND  $                  ; Flechas
     (v.ert EQ -1 OR v.ert EQ 6) THEN BEGIN
   v_ert   = 6
   IF TAG_EXIST(v,'sxm')    NE 0 OR TAG_EXIST(v,'sym') NE  0 THEN $
    MESSAGE, 'Arrows are incompatible with error bars'
   v_n2    = N_ELEMENTS(v.dx)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='dx',     STRUC_ADD={    dx:v.dx     + DBLARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible dx and y'
   v_n2    = N_ELEMENTS(v.dy)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='dy',     STRUC_ADD={    dy:v.dy     + DBLARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible dy and y'
   IF TAG_EXIST(v,'headl')  EQ 0 THEN v = CREATE_STRUCT(v, { headl:-1.0 + FLTARR(v.n)})
   v_n2    = N_ELEMENTS(v.headl)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='headl',  STRUC_ADD={ headl:v.headl  + FLTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible headl and y'
   IF TAG_EXIST(v,'arrowt') EQ 0 THEN v = CREATE_STRUCT(v, {arrowt:-1.0 + FLTARR(v.n)})
   v_n2    = N_ELEMENTS(v.arrowt)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='arrowt', STRUC_ADD={arrowt:v.arrowt + FLTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible arrowt and y'
   IF TAG_EXIST(v,'headt')  EQ 0 THEN v = CREATE_STRUCT(v, { headt:-1.0 + FLTARR(v.n)})
   v_n2    = N_ELEMENTS(v.headt)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='headt',  STRUC_ADD={ headt:v.headt  + FLTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible headt and y'
  ENDIF
; Texto
  IF TAG_EXIST(v,'txt') EQ   0 AND TAG_EXIST(v,'txm') NE  0 THEN $
   MESSAGE, 'Why does your structure have text modifiers but no text?'
  IF TAG_EXIST(v,'txt') NE   0 THEN BEGIN              ; Texto
   v_ert   = 7
   IF TAG_EXIST(v,'sxm') NE  0 OR TAG_EXIST(v,'sym') NE  0 OR TAG_EXIST(v,'dx') NE 0 THEN $
    MESSAGE, 'Text in structures is incompatible with error bars or arrows'
   v_n2    = N_ELEMENTS(v.txt)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='txt', STRUC_ADD={txt:v.txt + STRARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible txt and y'
   IF TAG_EXIST(v,'txm') EQ  0 THEN v = CREATE_STRUCT(v, {txm:'ALIGNMENT=0.5, /DATA, FONT=0' + STRARR(v.n)})
   v_n2    = N_ELEMENTS(v.txm)
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='txm', STRUC_ADD={txm:v.txm + STRARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible txm and y'
  ENDIF
; Aadimos por fin los valores "0" para los errores y ngulos
  IF TAG_EXIST(v,'sxm'   ) EQ  0 THEN v = CREATE_STRUCT(v, {   sxm:FLTARR(v.n)})
  IF TAG_EXIST(v,'sxp'   ) EQ  0 THEN v = CREATE_STRUCT(v, {   sxp:FLTARR(v.n)})
  IF TAG_EXIST(v,'sym'   ) EQ  0 THEN v = CREATE_STRUCT(v, {   sym:FLTARR(v.n)})
  IF TAG_EXIST(v,'syp'   ) EQ  0 THEN v = CREATE_STRUCT(v, {   syp:FLTARR(v.n)})
  IF TAG_EXIST(v,'ang'   ) EQ  0 THEN v = CREATE_STRUCT(v, {   ang:FLTARR(v.n)})
  IF TAG_EXIST(v,'dx'    ) EQ  0 THEN v = CREATE_STRUCT(v, {    dx:FLTARR(v.n)})
  IF TAG_EXIST(v,'dy'    ) EQ  0 THEN v = CREATE_STRUCT(v, {    dy:FLTARR(v.n)})
; Tipo de error
  IF v.ert NE -1 AND v.ert NE v_ert THEN MESSAGE, 'Error in data reprocessing'
  COPY_STRUC, v, v, EXCEPT_TAGS='ert', STRUC_ADD={ert:v_ert}
; Propiedades
  IF TAG_EXIST(v,'color')     NE  0 THEN BEGIN              ; Colores: slo [0] se usa pora lneas
   v_n2         = N_ELEMENTS(v.color    )
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='color',   STRUC_ADD={  color:v.color   + INTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible color and y'
  ENDIF ELSE BEGIN
   v            = CREATE_STRUCT(v, {    color:INTARR(v.n) + 0  })
  ENDELSE
  IF TAG_EXIST(v,'linestyle') NE  0 THEN BEGIN              ; Estilos de lnea (valor nico)
   v_n2         = N_ELEMENTS(v.linestyle)
   IF v_n2 NE 1    THEN MESSAGE, 'Structure with invalid linestyle'
  ENDIF ELSE BEGIN
   v            = CREATE_STRUCT(v, {linestyle:              0  })
  ENDELSE
  IF TAG_EXIST(v,'thick')     NE  0 THEN BEGIN              ; Grosor
   v_n2         = N_ELEMENTS(v.thick    )
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='thick',   STRUC_ADD={  thick:v.thick   + FLTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible thick and y'
  ENDIF ELSE BEGIN
   v            = CREATE_STRUCT(v, {    thick:FLTARR(v.n) + 1.0})
  ENDELSE
  IF TAG_EXIST(v,'psym')      NE  0 THEN BEGIN              ; Tipo de smbolo: slo [0] se usa pora lneas
   v_n2         = N_ELEMENTS(v.psym     )
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='psym',    STRUC_ADD={   psym:v.psym    + INTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible psym and y'
  ENDIF ELSE BEGIN
   v            = CREATE_STRUCT(v, {     psym:INTARR(v.n) + 0  })
  ENDELSE
  IF TAG_EXIST(v,'nofill')    NE  0 THEN BEGIN              ; Relleno de smbolo: no se usa pora lneas
   v_n2       = N_ELEMENTS(v.nofill   )
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='nofill',  STRUC_ADD={ nofill:v.nofill  + INTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible nofill and y'
  ENDIF ELSE BEGIN
   v            = CREATE_STRUCT(v, {   nofill:INTARR(v.n) + 0  })
  ENDELSE
  IF TAG_EXIST(v,'symsize')   NE  0 THEN BEGIN              ; Tamao de smbolo: no se usa pora lneas
   v_n2         = N_ELEMENTS(v.symsize  )
   IF v_n2 EQ 1 THEN COPY_STRUC, v, v, EXCEPT_TAGS='symsize', STRUC_ADD={symsize:v.symsize + FLTARR(v.n)} $
    ELSE IF  v_n2 NE v.n THEN MESSAGE, 'Structure with incompatible symsize and y'
  ENDIF ELSE BEGIN
   v            = CREATE_STRUCT(v, {  symsize:FLTARR(v.n) + 1.0})
  ENDELSE
 ENDELSE
; Arreglo de incompatibilidades de smbolos
 v.psym = v.psym*(v.psym NE   8) + 100*(v.psym EQ   8) 
 v.psym = v.psym*(v.psym NE  -8) - 100*(v.psym EQ  -8)
 v.psym = v.psym*(v.psym NE -10) +  10*(v.psym EQ -10)
 v.psym = v.psym*(v.psym NE -11) +  11*(v.psym EQ -11)
 v.psym = v.psym*(v.psym NE -12) +  12*(v.psym EQ -12)
 IF v.ert GE 1 AND v.ert LE 5 THEN FOR j=0,v.n-1 DO IF v.psym[j] LT 100 THEN v.psym[j] = 100
 IF (v.psym[0] GE 10 AND v.psym[0] LE 12) AND v.ert NE 0 THEN MESSAGE, 'Histograms cannot be error, arrow, or text plots'
 IF (v.psym[0] GE 10 AND v.psym[0] LE 12) AND v.n   LT 3 THEN MESSAGE, 'Histograms require at least three points'
 IF  v.psym[0] EQ 11                      THEN v.linestyle =           0
 IF  v.psym[0] EQ 11                      THEN v.thick     =   v.thick*0 + 1.0
 IF (v.psym[0] GE 10 AND v.psym[0] LE 12) THEN v.symsize   = v.symsize*0 + 3.0
; Aadido de informacin sobre los datos en s
 COPY_STRUC, v, v, EXCEPT_TAGS=['iplot','nplots'], STRUC_ADD={iplot:i, nplots:nplots}
 tmp  = EXECUTE('v' + vis + ' = v')
ENDFOR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;   (c) Inicializacin y clculo de ejes (XAXISL, YAXISL, XAXISU, YAXISU)      ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
IF KEYWORD_SET(XLOG)      +KEYWORD_SET(YLOG)      +KEYWORD_SET(EXTRAX)    +KEYWORD_SET(EXTRAY)    + $
   KEYWORD_SET(XTITLE)    +KEYWORD_SET(YTITLE)    +KEYWORD_SET(TICKF)                             + $
   KEYWORD_SET(XMORETICKS)+KEYWORD_SET(YMORETICKS)+KEYWORD_SET(XLESSTICKS)+KEYWORD_SET(YLESSTICKS)+ $
   KEYWORD_SET(EXTXTICKS) +KEYWORD_SET(EXTYTICKS) NE 0 AND KEYWORD_SET(QUIET) EQ 0 THEN $
   PRINT, 'Warning: using obsolete axis keywords!'
DOAXIS, xxl, 'xl',         LOG=xlog, EXTRA=extrax, MINMAX=minmax, TICKF=tickf, $
	           TITLE=xtitle, MORETICKS=xmoreticks, LESSTICKS=xlessticks,   $
	           EXTTICKS=extxticks, SPANISH=spanish
DOAXIS, yyl, 'yl',         LOG=ylog, EXTRA=extray, MINMAX=minmax, TICKF=tickf, $
	           TITLE=ytitle, MORETICKS=ymoreticks, LESSTICKS=ylessticks,   $
	           EXTTICKS=extyticks, SPANISH=spanish
DOAXIS, xxu, 'xu', VL=xxl, LOG=xlog, EXTRA=extrax, MINMAX=minmax, TICKF=tickf, $
	           TITLE=xtitle, MORETICKS=xmoreticks, LESSTICKS=xlessticks,   $
	           EXTTICKS=extxticks, SPANISH=spanish
DOAXIS, yyu, 'yu', VL=yyl, LOG=ylog, EXTRA=extray, MINMAX=minmax, TICKF=tickf, $
	           TITLE=ytitle, MORETICKS=ymoreticks, LESSTICKS=ylessticks,   $
	           EXTTICKS=extyticks, SPANISH=spanish
COPY_STRUC, xxl, xxl, EXCEPT_TAGS=['tickname'], STRUC_ADD={tickname:xxl.tickname2} ; This has to be done here to
COPY_STRUC, yyl, yyl, EXCEPT_TAGS=['tickname'], STRUC_ADD={tickname:yyl.tickname2} ; allow for separate upper/lower
COPY_STRUC, xxu, xxu, EXCEPT_TAGS=['tickname'], STRUC_ADD={tickname:xxu.tickname2} ; axis behavior and for corrections
COPY_STRUC, yyu, yyu, EXCEPT_TAGS=['tickname'], STRUC_ADD={tickname:yyu.tickname2} ; done to OUT
IF exit EQ 1 THEN GOTO, WRAPUP
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;          (d) Inicializacin de la grfica (PLOT y CTABLE)                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ENTRANCE1:
;
; Etiquetas no-PS de PLOT
;
IF KEYWORD_SET(XSIZE)+KEYWORD_SET(YSIZE)+KEYWORD_SET(XC)+KEYWORD_SET(YC)+ $
   KEYWORD_SET(NGX)  +KEYWORD_SET(TITLE)+KEYWORD_SET(AXISF) NE 0 AND KEYWORD_SET(QUIET) EQ 0 THEN $
   PRINT, 'Warning: using obsolete plot keywords!'
IF TAG_EXIST(pplot,'xsize')     EQ 0 THEN $
 IF KEYWORD_SET(XSIZE)               THEN pplot = CREATE_STRUCT(pplot, {    xsize:xsize            }) ELSE $
                                          pplot = CREATE_STRUCT(pplot, {    xsize:              0.0})
IF N_ELEMENTS(pplot.xsize)      NE 1 THEN MESSAGE, 'Incorrect dimensions for plot.xsize'
IF SIZE(pplot.xsize,/TYPE)      NE 4 AND $
   SIZE(pplot.xsize,/TYPE)      NE 5 THEN COPY_STRUC, pplot, pplot, EXCEPT_TAGS='xsize',     STRUC_ADD={    xsize:FLOAT(v.xsize    )}
IF TAG_EXIST(pplot,'ysize')     EQ 0 THEN $
 IF KEYWORD_SET(YSIZE)               THEN pplot = CREATE_STRUCT(pplot, {    ysize:ysize            }) ELSE $
                                          pplot = CREATE_STRUCT(pplot, {    ysize:              0.0})
IF N_ELEMENTS(pplot.ysize)      NE 1 THEN MESSAGE, 'Incorrect dimensions for plot.ysize'
IF SIZE(pplot.ysize,/TYPE)      NE 4 AND $
   SIZE(pplot.ysize,/TYPE)      NE 5 THEN COPY_STRUC, pplot, pplot, EXCEPT_TAGS='ysize',     STRUC_ADD={    ysize:FLOAT(v.ysize    )}
IF TAG_EXIST(pplot,'xc')        EQ 0 THEN $
 IF KEYWORD_SET(XC)                  THEN pplot = CREATE_STRUCT(pplot, {       xc:xc               }) ELSE $
                                          pplot = CREATE_STRUCT(pplot, {       xc:              0.0})
IF N_ELEMENTS(pplot.xc)         NE 1 THEN MESSAGE, 'Incorrect dimensions for plot.xc'
IF SIZE(pplot.xc,/TYPE)         NE 4 AND $
   SIZE(pplot.xc,/TYPE)         NE 5 THEN COPY_STRUC, pplot, pplot, EXCEPT_TAGS='xc',        STRUC_ADD={       xc:FLOAT(v.xc       )}
IF TAG_EXIST(pplot,'yc')        EQ 0 THEN $
 IF KEYWORD_SET(YC)                  THEN pplot = CREATE_STRUCT(pplot, {       yc:yc               }) ELSE $
                                          pplot = CREATE_STRUCT(pplot, {       yc:              0.0})
IF N_ELEMENTS(pplot.yc)         NE 1 THEN MESSAGE, 'Incorrect dimensions for plot.yc'
IF SIZE(pplot.yc,/TYPE)         NE 4 AND $
   SIZE(pplot.yc,/TYPE)         NE 5 THEN COPY_STRUC, pplot, pplot, EXCEPT_TAGS='yc',        STRUC_ADD={       yc:FLOAT(v.yc       )}
;IF TAG_EXIST(pplot,'ngx')       EQ 0 THEN $
; IF KEYWORD_SET(NGX)                 THEN pplot = CREATE_STRUCT(pplot, {      ngx:ngx              }) ELSE $
;                                          pplot = CREATE_STRUCT(pplot, {      ngx:                1})
;IF N_ELEMENTS(pplot.ngx)        NE 1 THEN MESSAGE, 'Incorrect dimensions for plot.ngx'
IF TAG_EXIST(pplot,'title')     EQ 0 THEN $
 IF KEYWORD_SET(TITLE)               THEN pplot = CREATE_STRUCT(pplot, {    title:title            }) ELSE $
                                          pplot = CREATE_STRUCT(pplot, {    title:               ''})
IF N_ELEMENTS(pplot.title)      NE 1 THEN MESSAGE, 'Incorrect dimensions for plot.title'
IF TAG_EXIST(pplot,'charsize')  EQ 0 THEN $
 IF KEYWORD_SET(AXISF)               THEN pplot = CREATE_STRUCT(pplot, { charsize:axisf            }) ELSE $
                                          pplot = CREATE_STRUCT(pplot, { charsize:              0.0})
IF N_ELEMENTS(pplot.charsize)   NE 1 THEN MESSAGE, 'Incorrect dimensions for plot.charsize'
IF SIZE(pplot.charsize,/TYPE)   NE 4 AND $
   SIZE(pplot.charsize,/TYPE)   NE 5 THEN COPY_STRUC, pplot, pplot, EXCEPT_TAGS='charsize',  STRUC_ADD={ charsize:FLOAT(v.charsize )}
IF TAG_EXIST(pplot,'charthick') EQ 0 THEN pplot = CREATE_STRUCT(pplot, {charthick:              0.0})
IF N_ELEMENTS(pplot.charthick)  NE 1 THEN MESSAGE, 'Incorrect dimensions for plot.charthick'
IF SIZE(pplot.charthick,/TYPE)  NE 4 AND $
   SIZE(pplot.charthick,/TYPE)  NE 5 THEN COPY_STRUC, pplot, pplot, EXCEPT_TAGS='charthick', STRUC_ADD={charthick:FLOAT(v.charthick)}
IF TAG_EXIST(pplot,'box')       EQ 0 THEN pplot = CREATE_STRUCT(pplot, {      box:[0.0,0.0,1.0,1.0]})
IF N_ELEMENTS(pplot.box)        NE 4 THEN MESSAGE, 'Incorrect dimensions for plot.box'
IF SIZE(pplot.charthick,/TYPE)  NE 4 AND $
   SIZE(pplot.charthick,/TYPE)  NE 5 THEN COPY_STRUC, pplot, pplot, EXCEPT_TAGS='box'      , STRUC_ADD={      box:FLOAT(v.box      )}
IF TAG_EXIST(pplot,'negative')  EQ 0 THEN pplot = CREATE_STRUCT(pplot, { negative:                0})
; CFILE
;
IF  N_ELEMENTS(CTABLE)  NE 0 OR N_ELEMENTS(CTEXT) NE 0 OR N_TAGS(cfile) NE 5 OR TAG_EXIST(cfile,'name') EQ 0 THEN BEGIN ; Se especifica parcialmente cfile 
 IF N_ELEMENTS(CTABLE)  NE 0 OR N_ELEMENTS(CTEXT) NE 0 THEN BEGIN
  IF KEYWORD_SET(QUIET) EQ 0 THEN PRINT, 'Using obsolete CFILE keywords!'
  IF cfile.name EQ 'colorjma.tbl' THEN BEGIN
   cdir  = jmaplot_path
  ENDIF ELSE BEGIN
   cpath = FILE_SEARCH(STRSPLIT(!DIR + '/resource/colors/:' + !PATH, PATH_SEP(/SEARCH_PATH),/EXTRACT) + '/' + cfile)
   IF STRLEN(cpath[0]) EQ 0 THEN STOP, cfile + ' not found in the path'
   cdir  = STRMID(cpath[0],0,STRLEN(cpath[0])-STRLEN(cfile))
  ENDELSE
  IF N_ELEMENTS(CTABLE) EQ 0 AND  cfile.name EQ 'colorjma.tbl' THEN ctable = 40
  IF N_ELEMENTS(CTABLE) EQ 0                                   THEN ctable =  0
  IF N_ELEMENTS(CTEXT)  EQ 0 THEN ctext = 0
  cfile  = CREATE_STRUCT(cfile, {dir:cdir, table:ctable, def:ctext, white:255})
 ENDIF ELSE BEGIN
  IF TAG_EXIST(cfile,'name' ) EQ 0 THEN cfile = CREATE_STRUCT(cfile, { name:'colorjma.tbl'})
  IF TAG_EXIST(cfile,'name' ) EQ 0 AND $
     cfile.name EQ 'colorjma.tbl'  THEN cfile = CREATE_STRUCT(cfile, {  dir:jmaplot_path  })
  IF TAG_EXIST(cfile,'dir'  ) EQ 0 THEN BEGIN
   cpath = FILE_SEARCH(STRSPLIT(!DIR + '/resource/colors/:' + !PATH, PATH_SEP(/SEARCH_PATH),/EXTRACT) + '/' + cfile.name)
   IF STRLEN(cpath[0]) EQ 0 THEN STOP, cfile.name + ' not found in the path'
   cfile = CREATE_STRUCT(cfile, {dir:STRMID(cpath[0],0,STRLEN(cpath[0])-STRLEN(cfile.name))})
  ENDIF
  IF TAG_EXIST(cfile,'table') EQ 0 AND $
     cfile.name EQ 'colorjma.tbl'  THEN cfile = CREATE_STRUCT(cfile, {table:            40})
  IF TAG_EXIST(cfile,'table') EQ 0 THEN cfile = CREATE_STRUCT(cfile, {table:             0})
  IF TAG_EXIST(cfile,'def'  ) EQ 0 THEN BEGIN
   IF cfile.name  EQ 'colorjma.tbl' AND $
      cfile.table EQ 38            THEN BEGIN
    IF pplot.negative         EQ 0 THEN cfile = CREATE_STRUCT(cfile, {  def:           254}) $
	                           ELSE cfile = CREATE_STRUCT(cfile, {  def:           255})
   ENDIF ELSE BEGIN
    IF pplot.negative         EQ 0 THEN cfile = CREATE_STRUCT(cfile, {  def:             0}) $
	                           ELSE cfile = CREATE_STRUCT(cfile, {  def:           255})
   ENDELSE
  ENDIF
  IF TAG_EXIST(cfile,'white') EQ 0 THEN BEGIN
   IF cfile.name  EQ 'colorjma.tbl' AND $
      cfile.table EQ 38            THEN BEGIN
    IF pplot.negative         EQ 0 THEN cfile = CREATE_STRUCT(cfile, {white:           255}) $
	                           ELSE cfile = CREATE_STRUCT(cfile, {white:           254})
   ENDIF ELSE BEGIN
    IF pplot.negative         EQ 0 THEN cfile = CREATE_STRUCT(cfile, {white:           255}) $
	                           ELSE cfile = CREATE_STRUCT(cfile, {white:             0})
   ENDELSE
  ENDIF
 ENDELSE
ENDIF ELSE IF N_TAGS(cfile) NE 5 THEN BEGIN                                                                    ; No se especifica nada de cfile
 IF cfile.name EQ 'colorjma.tbl' THEN BEGIN
  IF pplot.negative EQ 0 THEN cfile = CREATE_STRUCT(cfile, {dir:jmaplot_path, table:40, def:  0, white:255}) $
	                 ELSE cfile = CREATE_STRUCT(cfile, {dir:jmaplot_path, table:40, def:255, white:  0})
 ENDIF ELSE BEGIN
  IF pplot.negative EQ 0 THEN cfile = CREATE_STRUCT(cfile, {dir:jmaplot_path, table: 0, def:  0, white:255}) $
	                 ELSE cfile = CREATE_STRUCT(cfile, {dir:jmaplot_path, table: 0, def:255, white:  0})
 ENDELSE
ENDIF
OPENR, unit, cfile.dir + cfile.name, /GET_LUN, ERROR=err
IF err EQ 0 THEN BEGIN
 CLOSE, unit
 FREE_LUN, unit
ENDIF ELSE BEGIN
 MESSAGE, cfile.name + ' not found in ' + cfile.dir
ENDELSE
;
; Etiquetas PS de PLOT
;
IF TAG_EXIST(pplot,'psxsize')   EQ 0 THEN $
 IF KEYWORD_SET(PSXSIZE)             THEN pplot = CREATE_STRUCT(pplot, {  psxsize:psxsize  }) ELSE $
                                          pplot = CREATE_STRUCT(pplot, {  psxsize:      0.0})
IF N_ELEMENTS(pplot.psxsize)    NE 1 THEN MESSAGE, 'Incorrect dimensions for plot.psxsize'
IF SIZE(pplot.psxsize,/TYPE)    NE 4 AND $
   SIZE(pplot.psxsize,/TYPE)    NE 5 THEN COPY_STRUC, pplot, pplot, EXCEPT_TAGS='psxsize', STRUC_ADD={psxsize:FLOAT(pplot.psxsize)}
IF TAG_EXIST(pplot,'font')      EQ 0 THEN  pplot = CREATE_STRUCT(pplot, {     font:        0})
; IF KEYWORD_SET(PSFONTS)             THEN pplot = CREATE_STRUCT(pplot, {     font:        0}) ELSE $
;                                          pplot = CREATE_STRUCT(pplot, {     font:       -1})
IF N_ELEMENTS(pplot.font)       NE 1 THEN MESSAGE, 'Incorrect dimensions for plot.psfonts'
IF TAG_EXIST(pplot,'landscape') EQ 0 THEN $
 IF KEYWORD_SET(LANDSCAPE)           THEN pplot = CREATE_STRUCT(pplot, {landscape:landscape}) ELSE $
                                          pplot = CREATE_STRUCT(pplot, {landscape:        0})
IF N_ELEMENTS(pplot.landscape)  NE 1 THEN MESSAGE, 'Incorrect dimensions for plot.landscape'
IF psname NE '' THEN BEGIN
 PREPPS, psname, pplot, cfile, QUIET=quiet
ENDIF ELSE BEGIN
 IF pplot.psxsize  EQ 0 THEN MESSAGE,  'psxsize has to be set from multigraph'
 IF pplot.psaspect EQ 0 THEN MESSAGE, 'psaspect has to be set from multigraph'
 COPY_STRUC, pplot, pplot, EXCEPT_TAGS='psysize', STRUC_ADD={psysize:pplot.psxsize*pplot.psaspect}
ENDELSE
;
; Fijamos el tamao y posicin de la grfica.
;
GRPOS, pplot, xxl, yyl, xxu, yyu, QUIET=quiet
IF exit EQ 2 THEN GOTO, WRAPUP
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;          (e) Bucle de representacin de grficas.                            ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ENTRANCE2:
LOADCT, cfile.table, FILE=cfile.dir+cfile.name, SILENT=quiet
!X.CHARSIZE = xxl.charsize
!Y.CHARSIZE = yyl.charsize
!P.POSITION = pplot.box + [pplot.extraxl,pplot.extrayl,-pplot.extraxu,-pplot.extrayu]
!P.CHARSIZE = pplot.charsize
IF psname NE '' AND pplot.negative NE 0 THEN POLYFILL, [0.,1.,1.,0.], [0.,0.,1.,1.], /NORMAL, COLOR=cfile.white
PLOT, [xxl.range[0]+2*(xxl.range[1]-xxl.range[0])], $
      [yyl.range[0]+2*(yyl.range[1]-yyl.range[0])], $
      XRANGE=xxl.range, YRANGE=yyl.range,           $
      XLOG=xxl.log2, YLOG=yyl.log2,                 $
      XSTYLE=5, YSTYLE=5, /NOERASE
IF KEYWORD_SET(PREEXTERNAL) THEN FOR j=0,N_ELEMENTS(preexternal)-1 DO tmp = EXECUTE(preexternal[j])
IF KEYWORD_SET(BACKGROUND) THEN BEGIN
 sizeback = SIZE(background)
 IF sizeback[0] EQ 2 THEN BEGIN
  TV, background, !P.POSITION[0]*pplot.psxsize, !P.POSITION[1]*pplot.psysize, $
  XSIZE=(!P.POSITION[2]-!P.POSITION[0])*pplot.psxsize, $
  YSIZE=(!P.POSITION[3]-!P.POSITION[1])*pplot.psysize, $
  /CENTIMETERS
 ENDIF ELSE BEGIN
  LOADCT, 0, SILENT=quiet
  TV, $
  [[[background[*,*,0]]],[[background[*,*,1]]],[[background[*,*,2]]]], $
  TRUE=3, !P.POSITION[0]*pplot.psxsize, !P.POSITION[1]*pplot.psysize, $
  XSIZE=(!P.POSITION[2]-!P.POSITION[0])*pplot.psxsize, $
  YSIZE=(!P.POSITION[3]-!P.POSITION[1])*pplot.psysize, $
  /CENTIMETERS
  LOADCT, cfile.table, FILE=cfile.dir+cfile.name, SILENT=quiet
 ENDELSE
ENDIF
FOR i=0,nplots-1 DO BEGIN
 is       = STRTRIM(i+1,2)
 vvis     = 'vv' + is
 tmp      = EXECUTE('v = ' + vvis)
; [1] Caso de histograma 
 IF v.psym[0] GE 10 AND v.psym[0] LE 12 THEN BEGIN 
  IF i EQ 0 THEN OPLOT, [xxl.range[0]+2*(xxl.range[1]-xxl.range[0])], $
                        [yyl.range[0]+2*(yyl.range[1]-yyl.range[0])]
  aux    = 0.5*(v.x[0:v.n-2]+v.x[1:v.n-1])
  xxx    = FLTARR(2*v.n+3)
  yyy    = FLTARR(2*v.n+3)
  xxx[0:1]             = 2*aux[0]     - aux[1]
  xxx[2*v.n:2*v.n+1]   = 2*aux[v.n-2] - aux[v.n-3]
  xxx[2*v.n+2]         = 2*aux[0]     - aux[1]
  yyy[0]               = yyl.range[0]
  yyy[2*v.n+1:2*v.n+2] = yyl.range[0]
  FOR j=0L,v.n-1 DO BEGIN
   IF j NE v.n-1 THEN xxx[2*j+2:2*j+3] = aux[j]
   yyy[2*j+1:2*j+2] = v.y[j]
  ENDFOR
  xxxx = xxl.range[0]*(xxx LE xxl.range[0]) + xxx*(xxx GT xxl.range[0] AND xxx LT xxl.range[1]) + xxl.range[1]*(xxx GE xxl.range[1])
  yyyy = yyl.range[0]*(yyy LE yyl.range[0]) + yyy*(yyy GT yyl.range[0] AND yyy LT yyl.range[1]) + yyl.range[1]*(yyy GE yyl.range[1])
  IF v.psym[0] NE 10 THEN                                $
   POLYFILL, xxxx, yyyy, COLOR=v.color[0], /DATA
  colori = v.color[0]*(v.psym[0] EQ 10) +                $
            cfile.def*(v.psym[0] EQ 12)
  IF v.psym[0] EQ 10 THEN                                $
   OPLOT, xxx[1:2*v.n], yyy[1:2*v.n],                    $
         COLOR=colori, LINESTYLE=v.linestyle, PSYM=10,   $
	 THICK=v.thick[0]
; [2] Es caso de flechas 
 ENDIF ELSE IF v.ert EQ 6 THEN BEGIN
  FOR j=0,v.n-1 DO tmp = EXECUTE('ARROW2, v.x[j], v.y[j], v.x[j]+v.dx[j], v.y[j]+v.dy[j], ' + $
	                         'LA=v.headl[j], DX1=v.arrowt[j], DX2=v.headt[j], COL=v.color[j], /DATA')
; [3] Es caso de texto
 ENDIF ELSE IF v.ert EQ 7 THEN BEGIN
  FOR j=0,v.n-1 DO tmp = EXECUTE('XYOUTS, v.x[j], v.y[j], v.txt[j], ' + v.txm[j])
; [4] No es caso ni de texto ni de histograma
 ENDIF ELSE BEGIN
;     Hay lneas
  IF v.ert EQ 0 AND v.psym[0] LE 0 THEN BEGIN
   OPLOT, v.x, v.y, COLOR=v.color[0], LINESTYLE=v.linestyle, $
       THICK=v.thick[0]
  ENDIF
;     Hay smbolos
  IF MAX(ABS(v.psym)) NE 0 THEN BEGIN
   FOR j=0L,v.n-1 DO BEGIN
    IF   ABS(v.psym[j]) NE   0 THEN BEGIN
     IF  ABS(v.psym[j]) EQ   8 THEN GENSYM,                  0, $
	             NOFILL=v.nofill[j], THICK=v.thick[j] ELSE  $
     IF  ABS(v.psym[j]) GE 100 THEN GENSYM, ABS(v.psym[j])-100, $
                     NOFILL=v.nofill[j], THICK=v.thick[j]
     psymj = ABS(v.psym[j])*(ABS(v.psym[j]) LT 8) + $
	                  8*(ABS(v.psym[j]) GE 8)
     OPLOT, [v.x[j]], [v.y[j]], COLOR=v.color[j], PSYM=psymj,    $
           THICK=v.thick[j], SYMSIZE=v.symsize[j]
    ENDIF
   ENDFOR
  ENDIF
;     Hay errores
  IF v.ert NE 0 THEN PLOTERR2, v 
 ENDELSE
ENDFOR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;          (f) Representacin de ejes, marcas, ttulos y leyendas.             ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Ponemos los ttulos y los ejes
;
IF xxl.style NE -2 THEN AXIS2, pplot, xxl, yyl, cfile
IF yyl.style NE -2 THEN AXIS2, pplot, yyl, xxl, cfile
IF xxu.style NE -2 THEN AXIS2, pplot, xxu, yyl, cfile
IF yyu.style NE -2 THEN AXIS2, pplot, yyu, xxl, cfile
IF pplot.title NE '' THEN BEGIN
 titlen = strlen(pplot.title)
 IF titlen LE 60 THEN titsize=1.3*pplot.charsize ELSE titsize=1.1*pplot.charsize
 XYOUTS, 0.5*(!P.POSITION(0)+!P.POSITION(2)),                          $
         !P.POSITION(3)+pplot.extrayu-0.60*pplot.unitl/pplot.psaspect, $
	 pplot.title,                                                  $
         FONT=pplot.font, CHARSIZE=titsize, ALIGNMENT=0.5, /NORMAL, COLOR=cfile.def
ENDIF
;
; Comprobamos si se quiere escribir leyenda o no
;
IF legend.draw EQ 1 THEN BEGIN
 FOR i=0,N_ELEMENTS(legend.text)-1 DO IF legend.text[i] NE '' THEN GOTO, GOOD_LEGEND
 MESSAGE, 'Missing legend text'
 GOOD_LEGEND:
 IF TAG_EXIST(legend,'clear') EQ 0 THEN $
  IF KEYWORD_SET(LCLEAR)      EQ 0 THEN legend = CREATE_STRUCT(legend, {clear:     0}) $
                                   ELSE legend = CREATE_STRUCT(legend, {clear:     1})
 IF TAG_EXIST(legend,'xleft') EQ 0 THEN $
  IF KEYWORD_SET(LXLEFT)      EQ 0 THEN legend = CREATE_STRUCT(legend, {xleft:  0.05}) $
                                   ELSE legend = CREATE_STRUCT(legend, {xleft:lxleft})
 IF TAG_EXIST(legend,'ydown') EQ 0 THEN $
  IF KEYWORD_SET(LYDOWN)      EQ 0 THEN legend = CREATE_STRUCT(legend, {ydown:  0.05}) $
                                   ELSE legend = CREATE_STRUCT(legend, {ydown:lydown})
 IF TAG_EXIST(legend,'xsize') EQ 0 THEN $
  IF KEYWORD_SET(LXSIZE)      EQ 0 THEN legend = CREATE_STRUCT(legend, {xsize:  0.25}) $
                                   ELSE legend = CREATE_STRUCT(legend, {xsize:lxsize})
 IF TAG_EXIST(legend,'ysize') EQ 0 THEN $
  IF KEYWORD_SET(LYSIZE)      EQ 0 THEN legend = CREATE_STRUCT(legend, {ysize:  0.15}) $
                                   ELSE legend = CREATE_STRUCT(legend, {ysize:lysize})
 IF N_ELEMENTS(legend.xleft)  NE 1 THEN MESSAGE, 'Incorrect dimensions for legend.xleft'
 IF N_ELEMENTS(legend.ydown)  NE 1 THEN MESSAGE, 'Incorrect dimensions for legend.ydown'
 IF N_ELEMENTS(legend.xsize)  NE 1 THEN MESSAGE, 'Incorrect dimensions for legend.xsize'
 IF N_ELEMENTS(legend.ysize)  NE 1 THEN MESSAGE, 'Incorrect dimensions for legend.ysize'
  COPY_STRUC, legend, legend, EXCEPT_TAGS=['xleft','ydown','xsize','ysize'],  $
        STRUC_ADD={xleft:FLOAT(legend.xleft),ydown:FLOAT(legend.ydown), $
                   xsize:FLOAT(legend.xsize),ysize:FLOAT(legend.ysize)}
 color     = vv1.color[0]
 linestyle = vv1.linestyle
 psym      = vv1.psym[0]
 thick     = vv1.thick[0]
 IF N_ELEMENTS(vv1.symsize) EQ 1 OR STDDEV(vv1.symsize) EQ 0 THEN $
 symsize   = vv1.symsize[0] ELSE symsize = 1.0
 nofill    = vv1.nofill[0]
 ert       = vv1.ert
 FOR i=1,nplots-1 DO BEGIN
  is       = STRTRIM(i+1,2)
  vvis     = 'vv' + is
  tmp      = EXECUTE('color     = [color    ,' + vvis + '.color[0]  ]')
  tmp      = EXECUTE('linestyle = [linestyle,' + vvis + '.linestyle ]')
  tmp      = EXECUTE('psym      = [psym     ,' + vvis + '.psym[0]   ]')
  tmp      = EXECUTE('thick     = [thick    ,' + vvis + '.thick[0]  ]')
  tmp      = EXECUTE('IF N_ELEMENTS(vv1.symsize) EQ 1 OR STDDEV(vv1.symsize) EQ 0 THEN ' + $
                     'symsize   = [symsize  ,' + vvis + '.symsize[0]]             ELSE ' + $
                     'symsize   = [symsize  , 1.0]')
  tmp      = EXECUTE('nofill    = [nofill   ,' + vvis + '.nofill[0] ]')
  tmp      = EXECUTE('ert       = [ert      ,' + vvis + '.ert       ]')
 ENDFOR
 LEGEND, legend, pplot, cfile, color, linestyle, psym, symsize, thick, nofill, ert
ENDIF
IF KEYWORD_SET(EXTERNAL) THEN FOR i=0,N_ELEMENTS(external)-1 DO tmp = EXECUTE(external[i])
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;          (g) Cierre del fichero PostScript.                                  ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
IF psname NE '' THEN CLOSEPS
WRAPUP:
out    = {nplots:nplots, xaxisl:xxl, yaxisl:yyl, xaxisu:xxu, yaxisu:yyu, plot:pplot, cfile:cfile, legend:legend, $
          background:background, spanish:spanish, preexternal:preexternal, external:external, var:var}
FOR i=0,nplots-1 DO tmp = EXECUTE('out = CREATE_STRUCT(out, {v' + STRTRIM(i+1,2) + ':vv' + STRTRIM(i+1,2) + '})')
cfile  = cfileo
legend = legendo
!X     = xsv
!Y     = ysv
!P     = psv
END
;<PRE>
